#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>
Created on 2016-03-30, 10:11

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import


class PyMetaWearException(Exception):
    """Main exception."""
    pass


class PyMetaWearConnectionTimeout(PyMetaWearException):
    """Connection could not be established in time."""
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging
from ctypes import c_float, c_uint16

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.modules.base import PyMetaWearModule

log = logging.getLogger(__name__)


class HapticModule(PyMetaWearModule):
    """MetaWear Haptic module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, debug=False):
        super(HapticModule, self).__init__(board, debug)

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0}".format(self.module_name)

    def __repr__(self):
        return super(HapticModule, self).__repr__()

    @property
    def module_name(self):
        return 'Haptic'

    def notifications(self, callback=None):
        """No subscriptions possible for Haptic module.

        :raises: :py:exc:`~PyMetaWearException`

        """
        raise PyMetaWearException("Haptic module has no notifications.")

    def start_motor(self, duty_cycle_per, pulse_width_ms):
        """Activate the haptic motor.

        :param float duty_cycle_per: Strength of the motor,
            between [0, 100] percent
        :param int pulse_width_ms: How long to run the motor, in milliseconds

        """
        libmetawear.mbl_mw_haptic_start_motor(
            c_float(float(duty_cycle_per)),
            c_uint16(int(pulse_width_ms)))

    def start_buzzer(self, pulse_width_ms):
        """Activate the haptic buzzer.

        :param int pulse_width_ms: How long to run the motor, in milliseconds

        """
        libmetawear.mbl_mw_haptic_start_buzzer(
            c_uint16(int(pulse_width_ms)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`test_mwclient`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-06

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import pytest

import pymetawear.client
from .mock_backend import MockBackend
pymetawear.client.PyGattBackend = MockBackend
pymetawear.client.PyBluezBackend = MockBackend


@pytest.mark.parametrize("backend", ['pygatt', 'pybluez'])
@pytest.mark.parametrize("mw_board", range(7))
def test_client_init(backend, mw_board):
    MockBackend.boardType = mw_board
    c = pymetawear.client.MetaWearClient('XX:XX:XX:XX:XX:XX', backend=backend, debug=False)
    assert isinstance(c.backend, MockBackend)
    assert c.backend.boardType == mw_board
    assert c.backend.initialized
    assert c.backend.initialization_status == 0

    expected_cmds = [
        [0x01, 0x80], [0x02, 0x80], [0x03, 0x80], [0x04, 0x80],
        [0x05, 0x80], [0x06, 0x80], [0x07, 0x80], [0x08, 0x80],
        [0x09, 0x80], [0x0a, 0x80], [0x0b, 0x80], [0x0c, 0x80],
        [0x0d, 0x80], [0x0f, 0x80], [0x10, 0x80], [0x11, 0x80],
        [0x12, 0x80], [0x13, 0x80], [0x14, 0x80], [0x15, 0x80],
        [0x16, 0x80], [0x17, 0x80], [0x18, 0x80], [0x19, 0x80],
        [0xfe, 0x80], [0x0b, 0x84]
    ]
    assert c.backend.full_history == expected_cmds


@pytest.mark.parametrize("backend", ['pygatt', 'pybluez'])
@pytest.mark.parametrize("mw_board", range(7))
def test_client_init_delayed_connect(backend, mw_board):
    MockBackend.boardType = mw_board
    c = pymetawear.client.MetaWearClient('XX:XX:XX:XX:XX:XX', backend=backend, connect=False, debug=False)
    assert isinstance(c.backend, MockBackend)
    assert not c.backend.initialized
    c.connect()
    assert c.backend.boardType == mw_board
    assert c.backend.initialized
    assert c.backend.initialization_status == 0

    expected_cmds = [
        [0x01, 0x80], [0x02, 0x80], [0x03, 0x80], [0x04, 0x80],
        [0x05, 0x80], [0x06, 0x80], [0x07, 0x80], [0x08, 0x80],
        [0x09, 0x80], [0x0a, 0x80], [0x0b, 0x80], [0x0c, 0x80],
        [0x0d, 0x80], [0x0f, 0x80], [0x10, 0x80], [0x11, 0x80],
        [0x12, 0x80], [0x13, 0x80], [0x14, 0x80], [0x15, 0x80],
        [0x16, 0x80], [0x17, 0x80], [0x18, 0x80], [0x19, 0x80],
        [0xfe, 0x80], [0x0b, 0x84]
    ]
    assert c.backend.full_history == expected_cmds



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`accelerometer`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-10

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), 'pygatt', debug=True)
print("New client created: {0}".format(c))


def acc_callback(data):
    """Handle a (epoch, (x,y,z)) accelerometer tuple."""
    print("Epoch time: [{0}] - X: {1}, Y: {2}, Z: {3}".format(data[0], *data[1]))


print("Write accelerometer settings...")
c.accelerometer.set_settings(data_rate=100.0, data_range=4.0)
print("Subscribing to accelerometer signal notifications...")
c.accelerometer.high_frequency_stream = False
c.accelerometer.notifications(acc_callback)

time.sleep(20.0)

print("Unsubscribe to notification...")
c.accelerometer.notifications(None)

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-01

"""

from __future__ import division
from __future__ import print_function

from __future__ import absolute_import

import uuid


def high_low_2_uuid(uuid_high, uuid_low):
    """Combine high and low bits of a split UUID.

    :param uuid_high: The high 64 bits of the UUID.
    :type uuid_high: int
    :param uuid_low: The low 64 bits of the UUID.
    :type uuid_low: int
    :return: The UUID.
    :rtype: :py:class:`uuid.UUID`

    """
    return uuid.UUID(int=(uuid_high << 64) + uuid_low)


# Service specs obtained from connection.h
METAWEAR_SERVICE_NOTIFY_CHAR = (
    high_low_2_uuid(0x326a900085cb9195, 0xd9dd464cfbbae75a),
    high_low_2_uuid(0x326a900685cb9195, 0xd9dd464cfbbae75a)
)


# Service & char specs obtained from metawearboard.cpp

_DEVICE_INFO_SERVICE = high_low_2_uuid(0x0000180a00001000, 0x800000805f9b34fb)

METAWEAR_COMMAND_CHAR = (
    METAWEAR_SERVICE_NOTIFY_CHAR,
    high_low_2_uuid(0x326a900185cb9195, 0xd9dd464cfbbae75a)
)
DEV_INFO_FIRMWARE_CHAR = (
    _DEVICE_INFO_SERVICE,
    high_low_2_uuid(0x00002a2600001000, 0x800000805f9b34fb)
)
DEV_INFO_MODEL_CHAR = (
    _DEVICE_INFO_SERVICE,
    high_low_2_uuid(0x00002a2400001000, 0x800000805f9b34fb)
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PyGATT backend
--------------

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging

from ctypes import create_string_buffer
try:
    from pygatt import BLEAddressType
    from pygatt.backends.gatttool import gatttool
    _import_failure = None
except ImportError as e:
    _import_failure = e

from pymetawear.exceptions import PyMetaWearException, PyMetaWearConnectionTimeout
from pymetawear.compat import range_
from pymetawear.backends import BLECommunicationBackend

__all__ = ["PyGattBackend"]

log = logging.getLogger(__name__)


class PyGattBackend(BLECommunicationBackend):
    """
    Backend using `pygatt <https://github.com/peplin/pygatt>`_
    for BLE communication.
    """

    def __init__(self, address, interface=None, timeout=None, debug=False):
        if _import_failure is not None:
            raise PyMetaWearException(
                "pygatt[GATTTOOL] package error: {0}".format(_import_failure))

        self.name = 'pygatt'

        log.info("PyGattBackend: Creating new GATTToolBackend and starting GATTtool process...")
        self._backend = None

        if debug:
            log.setLevel(logging.DEBUG)

        super(PyGattBackend, self).__init__(
            address, interface,
            gatttool.DEFAULT_CONNECT_TIMEOUT_S if timeout is None else timeout, debug)

    @property
    def is_connected(self):
        if self._requester is not None:
            return self._requester._connected
        return False

    @property
    def requester(self):
        """Property handling the backend's device instance and its connection.

        :return: A connected ``pygatt`` BLE device instance.
        :rtype: :class:`pygatt.device.BLEDevice`

        """
        if self._requester is None:
            self.connect()

        if not self.is_connected:
            raise PyMetaWearConnectionTimeout(
                "PyGattBackend: Connection to {0} lost...".format(
                    self._address))

        return self._requester

    def connect(self, clean_connect=False):
        """Connect the GATTool process to the MetaWear board."""
        if self.is_connected:
            return

        self._backend = gatttool.GATTToolBackend(hci_device=self._interface)
        self._backend.start(reset_on_start=clean_connect)

        log.info("PyGattBackend: Connecting to {0} using GATTTool...".format(
            self._address))
        self._requester = self._backend.connect(
            self._address, timeout=self._timeout,
            address_type=BLEAddressType.random)

        super(PyGattBackend, self).connect()

    def disconnect(self):
        """Disconnect via the GATTTool process and terminate the
        interactive prompt.

        We can use the `stop` method since only one client can be
        connected to one GATTTool backend.

        """
        if self._backend is not None and self._backend:
            self._backend.stop()
        self._backend = None
        self._requester = None

    def _subscribe(self, characteristic_uuid, callback):
        return self.requester.subscribe(str(characteristic_uuid), callback)

    def read_gatt_char_by_uuid(self, characteristic):
        """Read the desired data from the MetaWear board using pygatt backend.

        :param GattCharacteristic characteristic: :class:`ctypes.POINTER` to a GattCharacteristic.
        :return: The read data.
        :rtype: str

        """
        return self.requester.char_read(str(self.get_uuid(characteristic)))

    def write_gatt_char_by_uuid(self, characteristic, command, length):
        """Write the desired data to the MetaWear board using pygatt backend.

        :param uuid.UUID characteristic: The UUID to the characteristic
            to write to.
        :param POINTER(c_ubyte) command: Data to send.
        :param int length: Number of characters in the command.

        """
        data_to_send = bytearray([command[i] for i in range_(length)])
        self.requester.char_write(str(self.get_uuid(characteristic)), data_to_send)

    def get_handle(self, characteristic_uuid, notify_handle=False):
        """Get handle from characteristic UUID.

        :param uuid.UUID characteristic_uuid: The UUID to find handle to.
        :param bool notify_handle: Set to true if subscription.
        :return: Integer handle.
        :rtype: int

        """
        return self.requester.get_handle(characteristic_uuid) + int(notify_handle)

    def _response_2_string_buffer(self, response):
        return create_string_buffer(bytes(response), len(response))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created on 2016-04-26

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging
from functools import wraps
from ctypes import cast, POINTER

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear import sensor
from pymetawear.mbientlab.metawear.core import DataTypeId, CartesianFloat
from pymetawear.modules.base import PyMetaWearModule, Modules

log = logging.getLogger(__name__)


def require_bmi160(f):
    def wrapper(*args, **kwargs):
        if getattr(args[0], 'gyro_class', None) is None:
            raise PyMetaWearException("There is not Gyroscope "
                                      "module of your MetaWear board!")
        return f(*args, **kwargs)

    return wrapper


class GyroscopeModule(PyMetaWearModule):
    """MetaWear gyroscope module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this gyroscope
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id, debug=False):
        super(GyroscopeModule, self).__init__(board, debug)
        self.module_id = module_id

        self.high_frequency_stream = False

        if self.module_id == Modules.MBL_MW_MODULE_NA:
            # No gyroscope present!
            self.gyro_class = None
        else:
            self.gyro_class = sensor.GyroBmi160

        if self.gyro_class is not None:
            # Parse possible output data rates for this accelerometer.
            self.odr = {int(re.search('^ODR_([0-9]+)HZ', k).groups()[0]):
                        getattr(self.gyro_class, k, None) for k in filter(
                lambda x: x.startswith('ODR'), vars(self.gyro_class))}
            self.fsr = {int(re.search('^FSR_([0-9]+)DPS', k).groups()[0]):
                        getattr(self.gyro_class, k, None) for k in
                        filter(lambda x: x.startswith('FSR'),
                               vars(self.gyro_class))}

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0} {1}: Data rates (Hz): {2}, Data ranges (dps): {3}".format(
            self.module_name, self.sensor_name,
            [float(k) for k in sorted(self.odr.keys(),
                                      key=lambda x: (float(x)))],
            [k for k in sorted(self.fsr.keys())])

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Gyroscope"

    @property
    def sensor_name(self):
        if self.gyro_class is not None:
            return self.gyro_class.__name__.replace('Gyro', '')
        else:
            return ''

    @property
    @require_bmi160
    def data_signal(self):
        if self.high_frequency_stream:
            return libmetawear.mbl_mw_gyro_bmi160_get_high_freq_rotation_data_signal(
                self.board)
        else:
            return libmetawear.mbl_mw_gyro_bmi160_get_rotation_data_signal(
                self.board)

    def _get_odr(self, value):
        sorted_ord_keys = sorted(self.odr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested ODR ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        k = int(sorted_ord_keys[diffs.index(min_diffs)])
        return self.odr.get(k)

    def _get_fsr(self, value):
        sorted_ord_keys = sorted(self.fsr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.1:
            raise ValueError(
                "Requested FSR ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted(self.fsr.keys())]))
        k = int(sorted_ord_keys[diffs.index(min_diffs)])
        return self.fsr.get(k)

    @require_bmi160
    def get_current_settings(self):
        raise NotImplementedError()

    @require_bmi160
    def get_possible_settings(self):
        return {
            'data_rate': [float(x) for x in sorted(
                self.odr.keys(), key=lambda x: (float(x)))],
            'data_range': [x for x in sorted(self.fsr.keys())]
        }

    @require_bmi160
    def set_settings(self, data_rate=None, data_range=None):
        """Set gyroscope settings.

         Can be called with two or only one setting:

         .. code-block:: python

            mwclient.gyroscope.set_settings(data_rate=200.0, data_range=8.0)

        will give the same result as

        .. code-block:: python

            mwclient.gyroscope.set_settings(data_rate=200.0)
            mwclient.gyroscope.set_settings(data_range=1000.0)

        albeit that the latter example makes two writes to the board.

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param float data_rate: The frequency of gyroscope updates in Hz.
        :param float data_range: The measurement range in the unit ``dps``,
            degrees per second.

        """
        if data_rate is not None:
            odr = self._get_odr(data_rate)
            if self._debug:
                log.debug("Setting Gyroscope ODR to {0}".format(odr))
            libmetawear.mbl_mw_gyro_bmi160_set_odr(self.board, odr)
        if data_range is not None:
            fsr = self._get_fsr(data_range)
            if self._debug:
                log.debug("Setting Gyroscope FSR to {0}".format(fsr))
            libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, fsr)

        if (data_rate is not None) or (data_range is not None):
            libmetawear.mbl_mw_gyro_bmi160_write_config(self.board)

    @require_bmi160
    def notifications(self, callback=None):
        """Subscribe or unsubscribe to gyroscope notifications.

        Convenience method for handling gyroscope usage.

        Example:

        .. code-block:: python

            def handle_notification(data):
                # Handle a (epoch_time, (x,y,z)) gyroscope tuple.
                epoch = data[0]
                xyz = data[1]
                print("[{0}] X: {1}, Y: {2}, Z: {3}".format(epoch, *xyz))

            mwclient.gyroscope.notifications(handle_notification)

        :param callable callback: Gyroscope notification callback function.
            If `None`, unsubscription to gyroscope notifications is registered.

        """
        if callback is None:
            self.stop()
            self.toggle_sampling(False)
            super(GyroscopeModule, self).notifications(None)
        else:
            super(GyroscopeModule, self).notifications(
                sensor_data(callback))
            self.toggle_sampling(True)
            self.start()

    @require_bmi160
    def start(self):
        """Switches the gyroscope to active mode."""
        libmetawear.mbl_mw_gyro_bmi160_start(self.board)

    @require_bmi160
    def stop(self):
        """Switches the gyroscope to standby mode."""
        libmetawear.mbl_mw_gyro_bmi160_stop(self.board)

    @require_bmi160
    def toggle_sampling(self, enabled=True):
        """Enables or disables gyroscope sampling.

        :param bool enabled: Desired state of the gyroscope.

        """
        if enabled:
            libmetawear.mbl_mw_gyro_bmi160_enable_rotation_sampling(self.board)
        else:
            libmetawear.mbl_mw_gyro_bmi160_disable_rotation_sampling(self.board)


def sensor_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.CARTESIAN_FLOAT:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(CartesianFloat))
            func((epoch, (data_ptr.contents.x,
                          data_ptr.contents.y,
                          data_ptr.contents.z)))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))

    return wrapper



from common import TestMetaWearBase
from ctypes import *
from mbientlab.metawear.core import Fn_VoidPtr, Fn_VoidPtr_Int, Status
from mbientlab.metawear.peripheral import Led
from mbientlab.metawear.processor import *
from mbientlab.metawear.sensor import* 
import threading

class TestSwitchLedController(TestMetaWearBase):
    def test_chain_setup(self):
        self.expected_cmds= [
            [0x09, 0x02, 0x01, 0x01, 0xff, 0x00, 0x02, 0x10],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x00, 0x09, 0x03, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x02, 0x02, 0x03, 0x0f],
            [0x0a, 0x03, 0x02, 0x02, 0x10, 0x10, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0xe8, 0x03, 0x00, 0x00, 0xff],
            [0x0a, 0x02, 0x09, 0x03, 0x02, 0x02, 0x01, 0x01],
            [0x0a, 0x03, 0x01],
            [0x0a, 0x02, 0x09, 0x03, 0x03, 0x02, 0x02, 0x01],
            [0x0a, 0x03, 0x01]
        ]

        self.cmds_recorded_handler= Fn_VoidPtr_Int(self.cmds_recorded)
        self.counter_handler= Fn_VoidPtr(self.counter_processor_created) 
        self.math_handler= Fn_VoidPtr(self.math_processor_created)

        switch_signal= self.libmetawear.mbl_mw_switch_get_state_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_counter_create(switch_signal, self.counter_handler)

    def counter_processor_created(self, signal):
        self.libmetawear.mbl_mw_dataprocessor_math_create(signal, Math.OPERATION_MODULUS, 2.0, self.math_handler)

    def math_processor_created(self, signal):
        self.modulus_signal= signal

        self.comparator_odd_handler= Fn_VoidPtr(self.comparator_odd_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(self.modulus_signal, Comparator.OPERATION_EQ, 
                1.0, self.comparator_odd_handler)

    def comparator_odd_created(self, signal):
        self.comp_odd_signal= signal

        self.comparator_even_handler= Fn_VoidPtr(self.comparator_even_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(self.modulus_signal, Comparator.OPERATION_EQ, 
                0.0, self.comparator_even_handler)

    def comparator_even_created(self, signal):
        pattern= Led.Pattern(pulse_duration_ms=1000, high_time_ms=500, high_intensity=16, low_intensity=16, 
                repeat_count=Led.REPEAT_INDEFINITELY)
        self.libmetawear.mbl_mw_event_record_commands(self.comp_odd_signal)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), Led.COLOR_BLUE)
        self.libmetawear.mbl_mw_led_play(self.board)
        self.libmetawear.mbl_mw_event_end_record(self.comp_odd_signal, self.commands_recorded_fn)

        self.libmetawear.mbl_mw_event_record_commands(signal)
        self.libmetawear.mbl_mw_led_stop_and_clear(self.board)
        self.libmetawear.mbl_mw_event_end_record(signal, self.cmds_recorded_handler)

    def cmds_recorded(self, event, status):
        self.assertEqual(self.command_history, self.expected_cmds)

class TestFreeFallDetectorRPro(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

    def test_freefall_setup(self):
        self.expected_cmds= [
            [0x09, 0x02, 0x03, 0x04, 0xff, 0xa0, 0x07, 0xa5, 0x01],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x20, 0x03, 0x05, 0x04],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x20, 0x0d, 0x09, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x02, 0x00, 0x06, 0x01, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff],
            [0x09, 0x02, 0x09, 0x03, 0x02, 0x00, 0x06, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00]
        ]

        self.rss_handler= Fn_VoidPtr(self.rss_processor_created) 
        accel_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_rss_create(accel_signal, self.rss_handler)

    def rss_processor_created(self, signal):
        self.avg_handler= Fn_VoidPtr(self.average_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_average_create(signal, 4, self.avg_handler)

    def average_processor_created(self, signal):
        self.threshold_handler= Fn_VoidPtr(self.threshold_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_threshold_create(signal, Threshold.MODE_BINARY, 0.5, 0.0,  
                self.threshold_handler)

    def threshold_processor_created(self, signal):
        self.threshold_signal= signal

        self.comparator_below_handler= Fn_VoidPtr(self.comparator_below_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(signal, Comparator.OPERATION_EQ, -1.0, 
                self.comparator_below_handler)

    def comparator_below_created(self, signal):
        self.comparator_above_handler= Fn_VoidPtr(self.comparator_above_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(self.threshold_signal, Comparator.OPERATION_EQ, 
                1.0, self.comparator_above_handler)

    def comparator_above_created(self, signal):
        self.assertEqual(self.command_history, self.expected_cmds)

class TestActivityMonitorRPro(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.rms_handler= Fn_VoidPtr(self.rms_processor_created) 
        accel_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_rms_create(accel_signal, self.rms_handler)

    def test_activity_setup(self):
        self.expected_cmds= [
            [0x09, 0x02, 0x03, 0x04, 0xff, 0xa0, 0x07, 0xa5, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x20, 0x02, 0x07],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x0f, 0x03],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x08, 0x03, 0x30, 0x75, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x03, 0x60, 0x0c, 0x0b, 0x00, 0x00, 0xc8, 0xaf],
            [0x09, 0x07, 0x03, 0x01],
            [0x09, 0x03, 0x01]
        ]

        self.assertEqual(self.command_history, self.expected_cmds)

    def test_time_processor_data(self):
        response= create_string_buffer(b'\x09\x03\x03\x4f\xee\xf4\x01', 7)
        expected= 2003.7236328125
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))
        self.assertAlmostEqual(self.data_float.value, expected)

    def test_time_processor_unsubscribe(self):
        expected= [0x09, 0x07, 0x03, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.time_signal)
        self.assertEqual(self.command, expected)

    def rms_processor_created(self, signal):
        self.accum_handler= Fn_VoidPtr(self.accum_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_accumulator_create_size(signal, 4, self.accum_handler)

    def accum_processor_created(self, signal):
        self.accum_signal= signal
        self.buffer_handler= Fn_VoidPtr(self.buffer_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_buffer_create(signal, self.buffer_handler)

    def buffer_processor_created(self, signal):
        self.time_handler= Fn_VoidPtr(self.time_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_time_create(self.accum_signal, Time.MODE_ABSOLUTE, 30000, self.time_handler)

    def time_processor_created(self, signal):
        self.time_signal= signal
        self.delta_handler= Fn_VoidPtr(self.delta_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_delta_create(signal, Delta.MODE_DIFFERENTIAL, 180000.0, self.delta_handler)

    def delta_processor_created(self, signal):
        self.libmetawear.mbl_mw_datasignal_subscribe(self.time_signal, self.sensor_data_handler)

class TestTemperatureConversionRPro(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.f_mult_handler= Fn_VoidPtr(self.f_mult_processor_created)
        self.temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, 
                MultiChannelTemperature.METAWEAR_RPRO_CHANNEL_ON_DIE)
        self.libmetawear.mbl_mw_dataprocessor_math_create(self.temp_signal, Math.OPERATION_MULTIPLY, 18.0, 
                self.f_mult_handler)

    def test_temperature_setup(self):
        self.expected_cmds= [
            [0x09, 0x02, 0x04, 0xc1, 0x00, 0x20, 0x09, 0x17, 0x02, 0x12, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x60, 0x09, 0x1f, 0x03, 0x0a, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x09, 0x1f, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x04, 0xc1, 0x00, 0x20, 0x09, 0x17, 0x01, 0x89, 0x08, 0x00, 0x00, 0x00],
            [0x09, 0x07, 0x02, 0x01],
            [0x09, 0x03, 0x01],
            [0x09, 0x07, 0x03, 0x01],
            [0x09, 0x03, 0x01]
        ]

        self.assertEqual(self.command_history, self.expected_cmds)

    def test_temperature_data(self):
        responses= [
            [create_string_buffer(b'\x04\x81\x00\x04\x01', 5), 32.5, "celsius"],
            [create_string_buffer(b'\x09\x03\x02\xd4\x02\x00\x00', 7), 90.5, "fahrenheit"],
            [create_string_buffer(b'\x09\x03\x03\x8d\x09\x00\x00', 7), 305.625, "kelvin"]
        ]

        for resp in responses:
            with self.subTest(response=resp[2]):
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, resp[0].raw, len(resp[0]))
                self.assertAlmostEqual(self.data_float.value, resp[1])

    def test_temperature_unsubscribe(self):
        expected_cmds= [
            [0x09, 0x07, 0x02, 0x00],
            [0x09, 0x07, 0x03, 0x00]
        ]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.temp_signal)
        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.fahrenheit_signal)
        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.kelvin_signal)

        unsubscribe_cmds= self.command_history[8:11].copy()
        self.assertEqual(unsubscribe_cmds, expected_cmds)

    def f_mult_processor_created(self, signal):
        self.f_divide_handler= Fn_VoidPtr(self.f_divide_processor_created) 
        self.libmetawear.mbl_mw_dataprocessor_math_create(signal, Math.OPERATION_DIVIDE, 10.0, 
                self.f_divide_handler)

    def f_divide_processor_created(self, signal):
        self.f_add_handler= Fn_VoidPtr(self.f_add_processor_created) 
        self.libmetawear.mbl_mw_dataprocessor_math_create(signal, Math.OPERATION_ADD, 32.0, self.f_add_handler)

    def f_add_processor_created(self, signal):
        self.fahrenheit_signal= signal;
        self.k_add_handler= Fn_VoidPtr(self.k_add_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_math_create(self.temp_signal, Math.OPERATION_ADD, 273.15, 
                self.k_add_handler)

    def k_add_processor_created(self, signal):
        self.kelvin_signal= signal

        self.libmetawear.mbl_mw_datasignal_subscribe(self.temp_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_datasignal_subscribe(self.fahrenheit_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_datasignal_subscribe(self.kelvin_signal, self.sensor_data_handler)

class TestDataCollector(TestMetaWearBase):
    def test_collector_setup(self):
        self.expected_cmds= [
            [0x09, 0x02, 0x05, 0xc7, 0x00, 0x20, 0x0a, 0x01, 0x10],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x20, 0x01, 0x02, 0x00, 0x00]
        ]

        self.sample_handler= Fn_VoidPtr(self.sample_processor_created)
        gpio_adc_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0, Gpio.ANALOG_READ_MODE_ADC)
        self.libmetawear.mbl_mw_dataprocessor_sample_create(gpio_adc_signal, 16, self.sample_handler)

    def sample_processor_created(self, signal):
        self.passthrough_handler= Fn_VoidPtr(self.passthrough_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_passthrough_create(signal, Passthrough.MODE_COUNT, 0, 
                self.passthrough_handler)

    def passthrough_processor_created(self, signal):
        self.assertEqual(self.command_history, self.expected_cmds)

class TestAccAxisProcessing(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.firmware_revision= create_string_buffer(b'1.2.3', 5)

        super().setUp()

        self.libmetawear.mbl_mw_acc_set_range(self.board, 16.0)
        self.acc_signal = self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)

    def test_acc_z_pulse_setup(self):
        expected_cmds= [
            [0x09, 0x02, 0x03, 0x04, 0xff, 0x24, 0x0b, 0x01, 0x00, 0x01, 0x00, 0x08, 0x00, 0x00, 0x10, 0x00],
            [0x09, 0x07, 0x00, 0x01],
            [0x09, 0x03, 0x01]
        ]

        acc_z_signal = self.libmetawear.mbl_mw_datasignal_get_component(self.acc_signal, Accelerometer.ACCEL_Z_AXIS_INDEX)
        self.libmetawear.mbl_mw_dataprocessor_pulse_create(acc_z_signal, Pulse.OUTPUT_AREA, 1.0, 16, 
                Fn_VoidPtr(lambda p: self.libmetawear.mbl_mw_datasignal_subscribe(p, self.sensor_data_handler)))

        self.assertEqual(self.command_history, expected_cmds)


    def test_acc_y_threshold(self):
        expected_cmds= [
            [0x09, 0x02, 0x03, 0x04, 0xff, 0x22, 0x0d, 0x0d, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x00, 0x06, 0x01, 0x01],
            [0x09, 0x07, 0x01, 0x01],
            [0x09, 0x03, 0x01]
        ]

        ths_created= Fn_VoidPtr(lambda t: self.libmetawear.mbl_mw_dataprocessor_comparator_create(t, Comparator.OPERATION_EQ, 1.0, comp_created))
        comp_created= Fn_VoidPtr(lambda c: self.libmetawear.mbl_mw_datasignal_subscribe(c, self.sensor_data_handler)) 
        acc_y_signal = self.libmetawear.mbl_mw_datasignal_get_component(self.acc_signal, Accelerometer.ACCEL_Y_AXIS_INDEX)
        self.libmetawear.mbl_mw_dataprocessor_threshold_create(acc_y_signal, Threshold.MODE_BINARY, 1.0, 0.0, ths_created)

        self.assertEqual(self.command_history, expected_cmds)

class TestGpioAdcPulse(TestMetaWearBase):
    def setUp(self):
        super().setUp()

        self.pulse_handler= Fn_VoidPtr(self.pulse_processor_created)
        gpio_adc_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0, Gpio.ANALOG_READ_MODE_ADC)
        self.libmetawear.mbl_mw_dataprocessor_pulse_create(gpio_adc_signal, Pulse.OUTPUT_PEAK, 500.0, 
                10, self.pulse_handler)

    def test_pulse_setup(self):
        self.expected_cmds= [
            [0x09, 0x02, 0x05, 0xc7, 0x00, 0x20, 0x0b, 0x01, 0x00, 0x02, 0xf4, 0x01, 0x00, 0x00, 0x0a, 0x00],
            [0x09, 0x07, 0x00, 0x01],
            [0x09, 0x03, 0x01]
        ]

        self.assertEqual(self.command_history, self.expected_cmds)

    def test_pulse_data(self):
        expected= 789
        response= create_string_buffer(b'\x09\x03\x00\x15\x03\x00\x00', 7)

        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))
        self.assertEqual(self.data_uint32.value, expected)

    def test_pulse_unsubscribe(self):
        expected= [0x09, 0x07, 0x00, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.pulse_signal)
        self.assertEqual(self.command, expected)

    def pulse_processor_created(self, signal):
        self.pulse_signal= signal
        self.libmetawear.mbl_mw_datasignal_subscribe(self.pulse_signal, self.sensor_data_handler)

class TestGpioFeedbackSetup(TestMetaWearBase):
    def setUp(self):
        super().setUp()

        self.passthrough_handler= Fn_VoidPtr(self.passthrough_processor_created)
        self.gpio_abs_ref_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0, 
                Gpio.ANALOG_READ_MODE_ABS_REF)
        self.libmetawear.mbl_mw_dataprocessor_passthrough_create(self.gpio_abs_ref_signal, Passthrough.MODE_COUNT, 
                0, self.passthrough_handler)

    def passthrough_processor_created(self, signal):
        self.offset_passthrough= signal
        self.math_handler= Fn_VoidPtr(self.math_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_math_create(self.gpio_abs_ref_signal, Math.OPERATION_SUBTRACT, 0.0, self.math_handler)

    def math_processor_created(self, signal):
        self.abs_ref_offset= signal
        self.gt_comparator_handler= Fn_VoidPtr(self.gt_comparator_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(signal, Comparator.OPERATION_GT, 0.0, 
                self.gt_comparator_handler)

    def gt_comparator_processor_created(self, signal):
        self.gt_comparator= signal
        self.gt_counter_handler= Fn_VoidPtr(self.gt_counter_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_counter_create(signal, self.gt_counter_handler)

    def gt_counter_processor_created(self, signal):
        self.gt_comparator_counter= signal
        self.gt_counter_comparator_handler= Fn_VoidPtr(self.gt_counter_comparator_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(signal, Comparator.OPERATION_EQ, 16.0, 
                self.gt_counter_comparator_handler)

    def gt_counter_comparator_processor_created(self, signal):
        self.gt_comparator_counter_comparator= signal
        self.lte_comparator_handler= Fn_VoidPtr(self.lte_comparator_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(self.abs_ref_offset, Comparator.OPERATION_LTE, 
                0.0, self.lte_comparator_handler)

    def lte_comparator_processor_created(self, signal):
        self.lte_comparator= signal
        self.lte_counter_handler= Fn_VoidPtr(self.lte_counter_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_counter_create(signal, self.lte_counter_handler)

    def lte_counter_processor_created(self, signal):
        self.lte_comparator_counter= signal
        self.lte_counter_comparator_handler= Fn_VoidPtr(self.lte_counter_comparator_processor_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(signal, Comparator.OPERATION_EQ, 16.0, 
                self.lte_counter_comparator_handler)

    def lte_counter_comparator_processor_created(self, signal):
        self.libmetawear.mbl_mw_event_record_commands(self.offset_passthrough)
        self.libmetawear.mbl_mw_dataprocessor_counter_set_state(self.lte_comparator_counter, 0)
        self.libmetawear.mbl_mw_dataprocessor_counter_set_state(self.gt_comparator_counter, 0)
        self.libmetawear.mbl_mw_dataprocessor_math_modify_rhs_signal(self.abs_ref_offset, self.offset_passthrough)
        self.libmetawear.mbl_mw_event_end_record(self.offset_passthrough, self.commands_recorded_fn)

        self.libmetawear.mbl_mw_event_record_commands(self.gt_comparator)
        self.libmetawear.mbl_mw_dataprocessor_counter_set_state(self.lte_comparator_counter, 0)
        self.libmetawear.mbl_mw_event_end_record(self.gt_comparator, self.commands_recorded_fn)

        self.libmetawear.mbl_mw_event_record_commands(self.gt_comparator_counter_comparator)
        self.libmetawear.mbl_mw_dataprocessor_passthrough_set_count(self.offset_passthrough, 1)
        self.libmetawear.mbl_mw_event_end_record(self.gt_comparator_counter_comparator, self.commands_recorded_fn)

        self.libmetawear.mbl_mw_event_record_commands(self.lte_comparator)
        self.libmetawear.mbl_mw_dataprocessor_counter_set_state(self.gt_comparator_counter, 0)
        self.libmetawear.mbl_mw_event_end_record(self.lte_comparator, self.commands_recorded_fn)

        self.libmetawear.mbl_mw_event_record_commands(signal)
        self.libmetawear.mbl_mw_dataprocessor_passthrough_set_count(self.offset_passthrough, 1)
        self.libmetawear.mbl_mw_event_end_record(signal, self.commands_recorded_fn)

class TestGpioFeedback(TestGpioFeedbackSetup):
    def test_feedback_setup(self):
        expected_cmds= [
            [0x09, 0x02, 0x05, 0xc6, 0x00, 0x20, 0x01, 0x02, 0x00, 0x00],
            [0x09, 0x02, 0x05, 0xc6, 0x00, 0x20, 0x09, 0x07, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x02, 0x60, 0x02, 0x1c],
            [0x09, 0x02, 0x09, 0x03, 0x03, 0x00, 0x06, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x05, 0x60, 0x02, 0x1c],
            [0x09, 0x02, 0x09, 0x03, 0x06, 0x00, 0x06, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x00, 0x09, 0x04, 0x05],
            [0x0a, 0x03, 0x06, 0x00, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x00, 0x09, 0x04, 0x05],
            [0x0a, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x00, 0x09, 0x05, 0x09, 0x05, 0x04],
            [0x0a, 0x03, 0x01, 0x09, 0x07, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x02, 0x09, 0x04, 0x05],
            [0x0a, 0x03, 0x06, 0x00, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x04, 0x09, 0x04, 0x03],
            [0x0a, 0x03, 0x00, 0x01, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x05, 0x09, 0x04, 0x05],
            [0x0a, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x07, 0x09, 0x04, 0x03],
            [0x0a, 0x03, 0x00, 0x01, 0x00]
        ]

        self.assertEqual(self.command_history, expected_cmds)

    def test_remove_out_of_order(self):
        expected_cmds= [
            [0x09, 0x06, 0x07],
            [0x0a, 0x04, 0x06],
            [0x09, 0x06, 0x06],
            [0x09, 0x06, 0x05],
            [0x0a, 0x04, 0x05],
            [0x09, 0x06, 0x04],
            [0x0a, 0x04, 0x04],
            [0x09, 0x06, 0x03],
            [0x09, 0x06, 0x02],
            [0x0a, 0x04, 0x03],
            [0x09, 0x06, 0x01]
        ]

        self.libmetawear.mbl_mw_dataprocessor_remove(self.lte_comparator)
        self.libmetawear.mbl_mw_dataprocessor_remove(self.abs_ref_offset)
        remove_cmds= self.command_history[22:].copy()

        self.assertEqual(remove_cmds, expected_cmds)

    def test_remove_passthrough(self):
        expected_cmds= [
            [0x09, 0x06, 0x00],
            [0x0a, 0x04, 0x00],
            [0x0a, 0x04, 0x01],
            [0x0a, 0x04, 0x02]
        ]

        self.libmetawear.mbl_mw_dataprocessor_remove(self.offset_passthrough)
        remove_cmds= self.command_history[22:].copy()

        self.assertEqual(remove_cmds, expected_cmds)

    def test_remove_math(self):
        expected_cmds= [
            [0x09, 0x06, 0x04],
            [0x0a, 0x04, 0x04],
            [0x09, 0x06, 0x03],
            [0x09, 0x06, 0x02],
            [0x0a, 0x04, 0x03],
            [0x09, 0x06, 0x07],
            [0x0a, 0x04, 0x06],
            [0x09, 0x06, 0x06],
            [0x09, 0x06, 0x05],
            [0x0a, 0x04, 0x05],
            [0x09, 0x06, 0x01]
        ]

        self.libmetawear.mbl_mw_dataprocessor_remove(self.abs_ref_offset)
        remove_cmds= self.command_history[22:].copy()

        self.assertEqual(remove_cmds, expected_cmds)

class TestPassthroughSetCount(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.processor_handler= Fn_VoidPtr(self.processor_created)
        self.baro_pa_signal= self.libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal(self.board)

    def set_count(self):
        self.status= self.libmetawear.mbl_mw_dataprocessor_passthrough_set_count(self.processors[0], 20)

    def test_valid_set_count(self):
        self.libmetawear.mbl_mw_dataprocessor_passthrough_create(self.baro_pa_signal, Passthrough.MODE_COUNT, 
                10, self.processor_created)
        self.set_count()
        self.assertEqual(self.status, Status.OK)

    def test_invalid_set_count(self):
        self.libmetawear.mbl_mw_dataprocessor_sample_create(self.baro_pa_signal, 16, self.processor_created)
        self.set_count()
        self.assertEqual(self.status, Status.WARNING_INVALID_PROCESSOR_TYPE)

class TestAccumulatorSetSum(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.processor_handler= Fn_VoidPtr(self.processor_created)
        self.baro_pa_signal= self.libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal(self.board)

    def set_state(self):
        self.status= self.libmetawear.mbl_mw_dataprocessor_set_accumulator_state(self.processors[0], 101325.0)

    def test_valid_set_state(self):
        self.libmetawear.mbl_mw_dataprocessor_accumulator_create(self.baro_pa_signal, self.processor_created)
        self.set_state()
        self.assertEqual(self.status, Status.OK)

    def test_invalid_set_count(self):
        self.libmetawear.mbl_mw_dataprocessor_time_create(self.baro_pa_signal, Time.MODE_DIFFERENTIAL, 30000, 
                self.processor_created)
        self.set_state()
        self.assertEqual(self.status, Status.WARNING_INVALID_PROCESSOR_TYPE)

class TestCounterSetCount(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.processor_handler= Fn_VoidPtr(self.processor_created)
        self.baro_pa_signal= self.libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal(self.board)

    def set_state(self):
        self.status= self.libmetawear.mbl_mw_dataprocessor_counter_set_state(self.processors[0], 128)

    def test_valid_set_state(self):
        self.libmetawear.mbl_mw_dataprocessor_counter_create(self.baro_pa_signal, self.processor_created)
        self.set_state()
        self.assertEqual(self.status, Status.OK)

    def test_invalid_set_count(self):
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(self.baro_pa_signal, Comparator.OPERATION_LT, 
                101325.0, self.processor_created)
        self.set_state()
        self.assertEqual(self.status, Status.WARNING_INVALID_PROCESSOR_TYPE)

class TestAverageReset(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.baro_pa_signal= self.libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal(self.board)

    def reset(self):
        self.status= self.libmetawear.mbl_mw_dataprocessor_average_reset(self.processors[0])

    def test_valid_reset(self):
        self.libmetawear.mbl_mw_dataprocessor_average_create(self.baro_pa_signal, 8, self.processor_created)
        self.reset()
        self.assertEqual(self.status, Status.OK)

    def test_invalid_reset(self):
        self.libmetawear.mbl_mw_dataprocessor_pulse_create(self.baro_pa_signal, Pulse.OUTPUT_AREA, 
                101325.0, 64, self.processor_created)
        self.reset()
        self.assertEqual(self.status, Status.WARNING_INVALID_PROCESSOR_TYPE)

class TestDeltaSetPrevious(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.baro_pa_signal= self.libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal(self.board)

    def set_reference(self):
        self.status= self.libmetawear.mbl_mw_dataprocessor_delta_set_reference(self.processors[0], 101325.0)

    def test_valid_set_previous(self):
        self.libmetawear.mbl_mw_dataprocessor_delta_create(self.baro_pa_signal, Delta.MODE_DIFFERENTIAL, 
                25331.25, self.processor_created)
        self.set_reference()
        self.assertEqual(self.status, Status.OK)

    def test_invalid_set_previous(self):
        self.libmetawear.mbl_mw_dataprocessor_math_create(self.baro_pa_signal, Math.OPERATION_DIVIDE, 1000.0, 
                self.processor_created)
        self.set_reference()
        self.assertEqual(self.status, Status.WARNING_INVALID_PROCESSOR_TYPE)

class TestThreshold(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()

    def test_valid_set_count(self):
        expected= [0x09, 0x02, 0x16, 0xc1, 0xff, 0x60, 0x0d, 0x0b, 0x00, 0xe4, 0x00, 0x00, 0x00, 0x00]

        signal= self.libmetawear.mbl_mw_humidity_bme280_get_percentage_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_threshold_create(signal, Threshold.MODE_BINARY, 57.0, 0.0, 
                Fn_VoidPtr(lambda p: self.assertEqual(self.command, expected)))

class TestThreeAxisRightShift(TestMetaWearBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD
        self.processor_created_fn= Fn_VoidPtr(self.processor_created)
        self.logger_created_fn= Fn_VoidPtr(self.logger_created)

    def test_acc_rshift(self):
        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_math_create(signal, Math.OPERATION_RSHIFT, 8, self.processor_created_fn)

    def processor_created(self, processor):
        self.libmetawear.mbl_mw_datasignal_log(processor, self.logger_created_fn)

    def logger_created(self, logger):
        expected_cmds= [
            [0x09, 0x02, 0x03, 0x04, 0xff, 0xa0, 0x09, 0x14, 0x08, 0x08, 0x00, 0x00, 0x00, 0x02],
            [0x0b, 0x02, 0x09, 0x03, 0x00, 0x40]
        ]
        self.assertEqual(self.command_history, expected_cmds)

# Future test data, figure out how to use this
#Shifted: (1.969, 0.812, 0.984)
#[126, 52, 63]
#Raw: [46, 126, 85, 52, 119, 63]
#Raw: (1.972, 0.818, 0.992)

class TestBmm150DataProcessor(TestMetaWearBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.boardType= TestMetaWearBase.METAWEAR_CPRO_BOARD

    def test_number_to_firmware(self):
        expected= [0x09, 0x02, 0x09, 0x03, 0x00, 0x20, 0x06, 0x00, 0x04, 0x00, 0xa0, 0x0f, 0x00, 0x00]

        comp_created_ptr= Fn_VoidPtr(lambda comp: self.assertEqual(self.command, expected))
        rss_created_ptr= Fn_VoidPtr(lambda rms: self.libmetawear.mbl_mw_dataprocessor_comparator_create(rms, Comparator.OPERATION_GT, 250.0, 
                comp_created_ptr))
        signal= self.libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_rss_create(signal, rss_created_ptr)

class TestDataProcessorTimeout(TestMetaWearBase):
    def dataprocessor_created(self, processor):
        self.created_proc= processor
        self.e.set()
        
    def commandLogger(self, board, characteristic, command, length):
        if (command[0] == 0x9 and command[1] == 0x2):
            response= create_string_buffer(b'\x0a\x00', 2)
            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))
        else:
            super().commandLogger(board, characteristic, command, length)

    def test_timeout_no_state(self):
        dataprocessor_created_fn= Fn_VoidPtr(self.dataprocessor_created)
        self.e= threading.Event()
        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_rss_create(signal, dataprocessor_created_fn)
        self.e.wait()

        self.assertIsNone(self.created_proc)

    def test_timeout_with_state(self):
        dataprocessor_created_fn= Fn_VoidPtr(self.dataprocessor_created)
        self.e= threading.Event()
        signal= self.libmetawear.mbl_mw_switch_get_state_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_accumulator_create(signal, dataprocessor_created_fn)
        self.e.wait()

        self.assertIsNone(self.created_proc)

class TestMultiComparatorRedirect(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.firmware_revision= create_string_buffer(b'1.2.3', 5)

        super().setUp()

    def test_modify(self):
        expected = [
            [0x09, 0x02, 0x03, 0x04, 0xFF, 0xA0, 0x07, 0xA5, 0x01],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x20, 0x06, 0x22, 0x00, 0x80],
            [0x09, 0x05, 0x01, 0x06, 0x22, 0x00, 0x20],
            [0x09, 0x05, 0x01, 0x06, 0x22, 0x00, 0x80]
        ]

        acc_signal = self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_dataprocessor_rss_create(acc_signal, self.processor_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_create(self.processors[0], Comparator.OPERATION_GT, 2.0, self.processor_created)
        self.libmetawear.mbl_mw_dataprocessor_comparator_modify(self.processors[1], Comparator.OPERATION_GT, 0.5)
        self.libmetawear.mbl_mw_dataprocessor_comparator_modify(self.processors[1], Comparator.OPERATION_GT, 2.0)

        self.assertEqual(self.command_history, expected)

class TestMultiComparator(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.firmware_revision= create_string_buffer(b'1.2.3', 5)

        super().setUp()

    def test_reference(self):
        expected= [0x09, 0x02, 0x05, 0xc7, 0x15, 0x20, 0x06, 0x6a, 0x00, 0x04, 0x00, 0x02, 0x00, 0x01, 0x80, 0x00]

        references= (c_float * 4)(1024, 512, 256, 128)
        adc_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0x15, Gpio.ANALOG_READ_MODE_ADC)

        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_create(adc_signal, Comparator.OPERATION_GTE, Comparator.MODE_REFERENCE, references, 
                len(references), self.processor_created)

        self.assertEqual(self.command, expected)

    def test_zone(self):
        expected= [0x09, 0x02, 0x14, 0x03, 0xff, 0x60, 0x06, 0x96, 0x00, 0xc4, 0x02, 0x00, 0xfa, 0x33, 0x5b, 0x00, 0x2a, 0x6d, 0x75, 0x01]

        references= (c_float * 3)(181248, 5977082, 24472874)
        signal= self.libmetawear.mbl_mw_als_ltr329_get_illuminance_data_signal(self.board)

        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_create(signal, Comparator.OPERATION_LT, Comparator.MODE_ZONE, references, 
                len(references), self.processor_created)

        self.assertEqual(self.command, expected)

    def test_absolute(self):
        expected= [0x09, 0x02, 0x04, 0xc1, 0x01, 0x20, 0x06, 0x03, 0xc0, 0x00, 0xc8, 0x00, 0xd0, 0x00]

        references= (c_float * 3)(24.0, 25.0, 26.0)
        signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, MultiChannelTemperature.METAWEAR_RPRO_CHANNEL_ON_BOARD_THERMISTOR)

        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_create(signal, Comparator.OPERATION_EQ, Comparator.MODE_ABSOLUTE, references, 
                len(references), self.processor_created)

        self.assertEqual(self.command, expected)

    def test_modify_config(self):
        references= (c_float * 4)(1024, 512, 256, 128)
        adc_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0x15, Gpio.ANALOG_READ_MODE_ADC)

        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_create(adc_signal, Comparator.OPERATION_GTE, Comparator.MODE_REFERENCE, references, 
                len(references), self.processor_created)

        expected= [0x09, 0x05, 0x00, 0x06, 0x52, 0x80, 0x00, 0x00, 0x01, 0x00, 0x02, 0x00, 0x04]
        references= (c_float * 4)(128, 256, 512, 1024)
        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_modify(self.processors[0], Comparator.OPERATION_LT, references, len(references))

        self.assertEqual(self.command, expected)

    def test_modify_config_signal(self):
        dummy = []
        expected_cmds= [
            [0x09, 0x02, 0x05, 0xc6, 0x00, 0x20, 0x06, 0x2a, 0x80, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x00, 0x09, 0x05, 0x05, 0x05, 0x03],
            [0x0a, 0x03, 0x00, 0x06, 0x2a, 0x00, 0x00]
        ]

        references= (c_float * 1)(128)
        adc_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0x0, Gpio.ANALOG_READ_MODE_ABS_REF)

        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_create(adc_signal, Comparator.OPERATION_GTE, Comparator.MODE_ABSOLUTE, references, 
                len(references), self.processor_created)

        self.libmetawear.mbl_mw_event_record_commands(self.processors[0])
        self.libmetawear.mbl_mw_dataprocessor_comparator_modify_signal(self.processors[0], Comparator.OPERATION_GTE, self.processors[0])
        self.libmetawear.mbl_mw_event_end_record(self.processors[0], Fn_VoidPtr_Int(lambda event, status: dummy.append(event)))

        self.assertEqual(self.command_history, expected_cmds)

class TestMathProcessor(TestMetaWearBase):
    def test_constant_op(self):
        expected= [0x09, 0x02, 0x04, 0xc1, 0x00, 0x20, 0x09, 0x17, 0x0b, 0xc8, 0x00, 0x00, 0x00, 0x00]

        temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, MultiChannelTemperature.METAWEAR_R_CHANNEL_ON_DIE)
        self.libmetawear.mbl_mw_dataprocessor_math_create(temp_signal, Math.OPERATION_CONSTANT, 25.0, 
                Fn_VoidPtr(lambda math: self.assertEqual(self.command, expected)))

    def test_constant_op_data(self):
        const_created_ptr= Fn_VoidPtr(self.const_created)
        temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, MultiChannelTemperature.METAWEAR_R_CHANNEL_ON_DIE)
        self.libmetawear.mbl_mw_dataprocessor_math_create(temp_signal, Math.OPERATION_CONSTANT, 25.0, const_created_ptr)

    def const_created(self, math):
        expected= 3.125
        response= create_string_buffer(b'\x09\x03\x00\x19\x00\x00\x00', 7)

        self.libmetawear.mbl_mw_datasignal_subscribe(math, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))
        self.assertEqual(self.data_float.value, expected)

class TestSensorFusionLimiter(TestMetaWearBase):
    def setUp(self):
        self.metawear_motion_r_services[0x09]= create_string_buffer(b'\x09\x80\x00\x01\x1c', 5)
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD

        super().setUp()

    def test_quaternion(self):
        expected = [
            [0x09, 0x02, 0x19, 0x07, 0xff, 0xe0, 0x08, 0x17, 0x14, 0x00, 0x00, 0x00],
            [0x0b, 0x02, 0x09, 0x03, 0x00, 0x60],
            [0x0b, 0x02, 0x09, 0x03, 0x00, 0x64],
            [0x0b, 0x02, 0x09, 0x03, 0x00, 0x68],
            [0x0b, 0x02, 0x09, 0x03, 0x00, 0x6c]
        ]

        signal = self.libmetawear.mbl_mw_sensor_fusion_get_data_signal(self.board, SensorFusion.DATA_QUATERION)
        self.libmetawear.mbl_mw_dataprocessor_time_create(signal, Time.MODE_ABSOLUTE, 20, self.processor_created)
        self.libmetawear.mbl_mw_datasignal_log(self.processors[0], self.logger_created)

        self.assertEqual(self.command_history, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import warnings
import logging
from functools import wraps
from ctypes import cast, POINTER, c_float

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import DataTypeId
from pymetawear.modules.base import PyMetaWearModule

log = logging.getLogger(__name__)

_CHANNEL_ID_TO_SOURCE_NAME = {
    -1: 'Invalid',
    0: 'On-Die',
    1: 'External',
    2: 'BMP280',
    3: 'On-Board'
}
_CHANNEL_ID_TO_SOURCE_NAME_4 = {
    0: 'On-Die',
    1: 'On-Board',
    2: 'External',
    3: 'BMP280'
}
_CHANNEL_ID_TO_SOURCE_NAME_2 = {
    0: 'On-Die',
    1: 'External',
}


class TemperatureModule(PyMetaWearModule):
    """MetaWear Temperature module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, debug=False):
        super(TemperatureModule, self).__init__(board, debug)

        self._active_channel = 0

        self.n_channels = int(
            libmetawear.mbl_mw_multi_chnl_temp_get_num_channels(self.board))
        self.channels = {}
        if self.n_channels == 4:
            self._channel_source_mapping = _CHANNEL_ID_TO_SOURCE_NAME_4
        elif self.n_channels == 2:
            self._channel_source_mapping = _CHANNEL_ID_TO_SOURCE_NAME_2
        else:
            self._channel_source_mapping = _CHANNEL_ID_TO_SOURCE_NAME

        self._reverse_channel_source_mapping = {
            v: k for k, v in self._channel_source_mapping.items()}

        for i in range(self.n_channels):
            source_enum = libmetawear.mbl_mw_multi_chnl_temp_get_source(
                self.board, i)
            self.channels[self._channel_source_mapping.get(source_enum)] = source_enum

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0}".format(self.module_name)

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Temperature"

    @property
    def data_signal(self):
        return libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(
            self.board, self._active_channel)

    def configure_external_thermistor(self, channel_id, data_pin,
                                      pulldown_pin, active_high):
        """Configure the external thermistor.

        :param channel_id: Channel ID of the external thermistor
        :param int data_pin: GPIO pin reading the data
        :param int pulldown_pin: GPIO pin the pulldown resistor is connected to
        :param int active_high: Zero if the pulldown pin is not active high,
            non-zero if active high

        """
        libmetawear.mbl_mw_multi_chnl_temp_configure_ext_thermistor(
            self.board, channel_id, data_pin, pulldown_pin, active_high)

    def get_possible_settings(self):
        """Get the possible settings for this module.

        :return: Dict with the names of the possible channel choices.

        """
        return {
            'channel': [c for c in sorted(
                self.channels.keys(), key=lambda x: self.channels[x])],
        }

    def get_current_settings(self):
        """Get the current settings for this module.

        :return: Dict with the name of the currently active channel.

        """
        return {
            'channel': _CHANNEL_ID_TO_SOURCE_NAME.get(self._active_channel)
        }

    def set_settings(self, channel=None):
        """Set temperature channel settings.

         .. code-block:: python

            mwclient.temperature.set_settings(channel='On-Die')

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param str channel: The name of the temperature channel to make active.

        """
        if channel is not None:
            if channel in self.channels:
                self._active_channel = self._reverse_channel_source_mapping.get(channel)
            else:
                warnings.warn("Desired channel {0} was not one of the possible ones: {1}.".format(
                    channel, self.get_possible_settings().get('channel')))

    def read_temperature(self):
        """Triggers a temperature notification.

        N.B. that a :meth:`~notifications` call that registers a
        callback for temperature should have been done prior to calling
        this method.

        """
        if self.callback is None:
            warnings.warn("No temperature callback is registered!", RuntimeWarning)
        libmetawear.mbl_mw_datasignal_read(self.data_signal)

    def notifications(self, callback=None):
        """Subscribe or unsubscribe to temperature notifications.

        Convenience method for handling temperature usage.

        Example:

        .. code-block:: python

            def temperature_callback(data):
                epoch = data[0]
                temp = data[1]
                print("[{0}] Temperature {1}".format(epoch, temp))

            mwclient.temperature_func.notifications(temperature_callback)

        :param callable callback: Temperature notification callback function.
            If `None`, unsubscription to temperature_func notifications is registered.

        """
        super(TemperatureModule, self).notifications(
            temperature_func(callback) if callback is not None else None)


def temperature_func(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.FLOAT:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(c_float))
            func((epoch, data_ptr.contents.value))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))
    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`led`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), 'pybluez', debug=True)
print("New client created: {0}".format(c))


def switch_callback(data):
    """Handle a (epoch, status) switch tuple."""
    epoch = data[0]
    status = data[1]
    print("[{0}] Switch {1}!".format(
        epoch, 'pressed' if status else 'released'))


# Create subscription
c.switch.notifications(switch_callback)

time.sleep(10.0)

# Remove subscription
c.switch.notifications(None)
time.sleep(1.0)

c.disconnect()



from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.sensor import HumidityBme280

class TestHumidityBme280Config(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()

    def test_oversampling(self):
        tests= [
            {
                'expected': [0x16, 0x02, 0x01],
                'oversampling': HumidityBme280.OVERSAMPLING_1X,
                'oversampling_name': '1x'
            },
            {
                'expected': [0x16, 0x02, 0x02],
                'oversampling': HumidityBme280.OVERSAMPLING_2X,
                'oversampling_name': '2x'
            },
            {
                'expected': [0x16, 0x02, 0x03],
                'oversampling': HumidityBme280.OVERSAMPLING_4X,
                'oversampling_name': '4x'
            },
            {
                'expected': [0x16, 0x02, 0x04],
                'oversampling': HumidityBme280.OVERSAMPLING_8X,
                'oversampling_name': '8x'
            },
            {
                'expected': [0x16, 0x02, 0x05],
                'oversampling': HumidityBme280.OVERSAMPLING_16X,
                'oversampling_name': '16x'
            },
        ]

        for test in tests:
            with self.subTest(channel= test['oversampling_name']):
                self.libmetawear.mbl_mw_humidity_bme280_set_oversampling(self.board, test['oversampling'])
                self.assertEqual(self.command, test['expected'])

class TestHumidityBme280Data(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()

    def test_humidity_data(self):
        expected= 63.1943359375
        response= create_string_buffer(b'\x16\x81\xc7\xfc\x00\x00')

        signal= self.libmetawear.mbl_mw_humidity_bme280_get_percentage_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))

        self.assertAlmostEqual(self.data_float.value, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`led`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
from pymetawear.client import MetaWearClient

address_1 = 'DD:3A:7D:4D:56:F0'
address_2 = 'FF:50:35:82:3B:5A'

print("Connect to {0}...".format(address_1))
client_1 = MetaWearClient(str(address_1), timeout=10.0, debug=False)
print("New client created: {0}".format(client_1))
print("Connect to {0}...".format(address_2))
client_2 = MetaWearClient(str(address_2), timeout=10.0, debug=False)
print("New client created: {0}".format(client_2))

print("Blinking 10 times with green LED on client 1...")
pattern = client_1.led.load_preset_pattern('blink', repeat_count=10)
client_1.led.write_pattern(pattern, 'g')
client_1.led.play()

print("Blinking 10 times with red LED on client 2...")
pattern = client_2.led.load_preset_pattern('blink', repeat_count=10)
client_2.led.write_pattern(pattern, 'r')
client_2.led.play()

time.sleep(5.0)

client_1.disconnect()
client_2.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`gyroscope`
===============

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-26

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
from ctypes import cast, POINTER

from pymetawear.client import MetaWearClient, libmetawear
from pymetawear.discover import select_device
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import \
    CartesianFloat, DataTypeId, Fn_DataPtr


address = select_device()
c = MetaWearClient(str(address), 'pygatt', debug=True)
print("New client created: {0}".format(c))


def _callback(data):
    """Handle a (x,y,z) gyroscope tuple."""
    if data.contents.type_id == DataTypeId.CARTESIAN_FLOAT:
        data_ptr = cast(data.contents.value, POINTER(CartesianFloat))
        print("X: {0}, Y: {1}, Z: {2}".format(*(data_ptr.contents.x,
                                     data_ptr.contents.y,
                                     data_ptr.contents.z)))
    else:
        raise PyMetaWearException('Incorrect data type id: ' +
            str(data.contents.type_id))
the_callback = Fn_DataPtr(_callback)


print("Set gyroscope settings...")
libmetawear.mbl_mw_gyro_bmi160_set_odr(c.board, 9)
libmetawear.mbl_mw_gyro_bmi160_set_range(c.board, 1)
print("Write gyroscope config....")
libmetawear.mbl_mw_gyro_bmi160_write_config(c.board)
print("Subscribing to gyroscope signal...")
data_signal = libmetawear.mbl_mw_gyro_bmi160_get_rotation_data_signal(c.board)
libmetawear.mbl_mw_datasignal_subscribe(data_signal, the_callback)
print("Enable gyro sampling on board...")
libmetawear.mbl_mw_gyro_bmi160_enable_rotation_sampling(c.board)
print("Start gyro on board...")
libmetawear.mbl_mw_gyro_bmi160_start(c.board)

time.sleep(20.0)

print("Stop gyroscope...")
libmetawear.mbl_mw_gyro_bmi160_stop(c.board)
print("Disable gyroscope sampling...")
libmetawear.mbl_mw_gyro_bmi160_disable_rotation_sampling(c.board)
print("Unsubscribe to notification...")
libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)



time.sleep(5.0)

c.disconnect()



from common import TestMetaWearBase
from ctypes import byref, create_string_buffer
from mbientlab.metawear.sensor import I2C

class TestI2cR(TestMetaWearBase):
    def test_read_who_am_i(self):
        expected= [0x0d, 0xc1, 0x1c, 0x0d, 0x0a, 0x01]

        parameters= I2C.ReadParameters(device_addr= 0x1c, register_addr= 0xd)
        signal= self.libmetawear.mbl_mw_i2c_get_data_signal(self.board, 1, 0xa)
        self.libmetawear.mbl_mw_datasignal_read_with_parameters(signal, byref(parameters))
        self.assertEqual(self.command, expected)

    def test_who_am_i_data(self):
        expected= [0x2a]
        response= create_string_buffer(b'\x0d\x81\x0a\x2a', 4)

        signal= self.libmetawear.mbl_mw_i2c_get_data_signal(self.board, 1, 0xa)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))

        self.assertEqual(self.data_byte_array, expected)



from common import TestMetaWearBase
from ctypes import byref, cast, create_string_buffer, c_ubyte, c_uint, POINTER
from logdata import *
from mbientlab.metawear.core import *
from mbientlab.metawear.processor import *
from mbientlab.metawear.sensor import Gpio
from test_dataprocessor import TestGpioFeedbackSetup
from test_logging import TestAccelerometerLoggingBase, TestGyroYAxisLoggingBase
import serializedstate
import threading

class TestMetaWearBoard(TestMetaWearBase):
    def test_no_response_handler(self):
        tests= [
            {
                'response': create_string_buffer(b'\x03\x04\x16\xc4\x94\xa2\x2a\xd0', 8),
                'status': Status.WARNING_UNEXPECTED_SENSOR_DATA,
                'name': 'acc'
            },
            {
                'response': create_string_buffer(b'\x14\x03\xed\x92\xb0\x00', 6),
                'status': Status.WARNING_INVALID_RESPONSE,
                'name': 'ambient_light'
            },
            {
                'response': create_string_buffer(b'\x12\x01\xd3\x35\x8b\x01', 6),
                'status': Status.WARNING_INVALID_RESPONSE,
                'name': 'baro_pressure'
            },
            {
                'response': create_string_buffer(b'\x12\x02\x1e\x1f\xfe\xff', 6),
                'status': Status.WARNING_INVALID_RESPONSE,
                'name': 'baro_altitude'
            },
            {
                'response': create_string_buffer(b'\x13\x05\x3e\x43\xff\x7f\x00\x80', 8),
                'status': Status.WARNING_INVALID_RESPONSE,
                'name': 'gyro'
            },
            {
                'response': create_string_buffer(b'\x01\x01\x01', 3),
                'status': Status.WARNING_UNEXPECTED_SENSOR_DATA,
                'name': 'switch'
            }
        ]

        for test in tests:
            with self.subTest(response=test['name']):
                status= self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, test['response'].raw, len(test['response'].raw))
                self.assertEqual(status, test['status'])

    def test_service_discovery(self):
        self.assertEqual(self.init_status, Status.OK)

class TestMetaWearBoardInitialize(TestMetaWearBase):
    def test_reinitialize(self):
        expected_cmds= [
            [0x01, 0x80], [0x02, 0x80], [0x03, 0x80], [0x04, 0x80],
            [0x05, 0x80], [0x06, 0x80], [0x07, 0x80], [0x08, 0x80],
            [0x09, 0x80], [0x0a, 0x80], [0x0b, 0x80], [0x0c, 0x80],
            [0x0d, 0x80], [0x0f, 0x80], [0x10, 0x80], [0x11, 0x80],
            [0x12, 0x80], [0x13, 0x80], [0x14, 0x80], [0x15, 0x80],
            [0x16, 0x80], [0x17, 0x80], [0x18, 0x80], [0x19, 0x80],
            [0xfe, 0x80], [0x0b, 0x84]
        ]

        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)
        self.assertEqual(self.full_history, expected_cmds)

    def test_reinitialize_new_firmware(self):
        expected_cmds= [
            [0x01, 0x80], [0x02, 0x80], [0x03, 0x80], [0x04, 0x80],
            [0x05, 0x80], [0x06, 0x80], [0x07, 0x80], [0x08, 0x80],
            [0x09, 0x80], [0x0a, 0x80], [0x0b, 0x80], [0x0c, 0x80],
            [0x0d, 0x80], [0x0f, 0x80], [0x10, 0x80], [0x11, 0x80],
            [0x12, 0x80], [0x13, 0x80], [0x14, 0x80], [0x15, 0x80], 
            [0x16, 0x80], [0x17, 0x80], [0x18, 0x80],
            [0xfe, 0x80], [0x0b, 0x84],
            [0x01, 0x80], [0x02, 0x80], [0x03, 0x80], [0x04, 0x80],
            [0x05, 0x80], [0x06, 0x80], [0x07, 0x80], [0x08, 0x80],
            [0x09, 0x80], [0x0a, 0x80], [0x0b, 0x80], [0x0c, 0x80],
            [0x0d, 0x80], [0x0f, 0x80], [0x10, 0x80], [0x11, 0x80],
            [0x12, 0x80], [0x13, 0x80], [0x14, 0x80], [0x15, 0x80], 
            [0x16, 0x80], [0x17, 0x80], [0x18, 0x80],
            [0xfe, 0x80], [0x0b, 0x84]
        ]

        self.firmware_revision= create_string_buffer(b'1.1.4', 5)
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)
        self.assertEqual(self.full_history, expected_cmds)

class TestMetaWearBoardInitialize(TestMetaWearBase):
    def setUp(self):
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

    def initialized(self, board, status):
        super().initialized(board, status)

        self.e.set()

    def commandLogger(self, board, characteristic, command, length):
        if (command[1] == 0x80):
            response= create_string_buffer(b'\x00\x00', 2)
            response[0]= command[0]
            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))
        else:
            super().commandLogger(board, characteristic, command, length)

    def test_timeout(self):
        self.e= threading.Event()
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)
        self.e.wait()

        self.assertEqual(self.init_status, Status.ERROR_TIMEOUT)

class TestTearDown(TestGpioFeedbackSetup):
    def test_dataproc_event(self):
        expected_cmds= [
            [0x09, 0x08],
            [0x0a, 0x05],
            [0x0b, 0x0a]
        ]

        self.libmetawear.mbl_mw_metawearboard_tear_down(self.board)
        tear_down_cmds= self.command_history[22:].copy()

        self.assertEqual(tear_down_cmds, expected_cmds)

class TestTimerTearDown(TestMetaWearBase):
    def test_timer(self):
        expected_cmds= [
            [0x0c, 0x05, 0x00],
            [0x0c, 0x05, 0x01],
            [0x0c, 0x05, 0x02],
            [0x0c, 0x05, 0x03],
            [0x09, 0x08],
            [0x0a, 0x05],
            [0x0b, 0x0a]
        ]

        self.libmetawear.mbl_mw_timer_create(self.board, 667408, -1, 0, self.timer_signal_ready)
        self.libmetawear.mbl_mw_timer_create(self.board, 1000, -1, 0, self.timer_signal_ready)
        self.libmetawear.mbl_mw_timer_create(self.board, 1000, 10, 0, self.timer_signal_ready)
        self.libmetawear.mbl_mw_timer_create_indefinite(self.board, 1000, 0, self.timer_signal_ready)

        self.libmetawear.mbl_mw_metawearboard_tear_down(self.board)
        self.assertEqual(self.command_history[4:], expected_cmds)

class TestMetaWearBoardSerialize(TestMetaWearBase):
    motion_r_state = [
        0x01, 
        0x01, 0x03, 0x01, 
        0x01, 0x35, 
        0x19, 
        0x01, 0x00, 0x00, 0x00, 
        0x02, 0x00, 0x00, 0x00, 
        0x03, 0x01, 0x01, 0x00, 
        0x04, 0x01, 0x00, 0x04, 0x00, 0x03, 0x01, 0x02, 
        0x05, 0x00, 0x01, 0x05, 0x03, 0x03, 0x03, 0x03, 0x01, 
        0x06, 0x00, 0x00, 0x00, 
        0x07, 0x00, 0x00, 0x00, 
        0x08, 0x00, 0x00, 0x00, 
        0x09, 0x00, 0x00, 0x01, 0x1c, 
        0x0a, 0x00, 0x00, 0x01, 0x1c, 
        0x0b, 0x00, 0x02, 0x05, 0x08, 0x80, 0x2b, 0x00, 0x00, 
        0x0c, 0x00, 0x00, 0x01, 0x08, 
        0x0d, 0x00, 0x01, 0x00, 
        0x0f, 0x00, 0x01, 0x01, 0x08, 
        0x10, 0xff, 0xff, 
        0x11, 0x00, 0x03, 0x00, 
        0x12, 0x00, 0x00, 0x00, 
        0x13, 0x00, 0x01, 0x00, 
        0x14, 0x00, 0x00, 0x00, 
        0x15, 0x00, 0x01, 0x00, 
        0x16, 0xff, 0xff, 
        0x17, 0xff, 0xff, 
        0x18, 0xff, 0xff, 
        0x19, 0x00, 0x00, 0x08, 0x03, 0x00, 0x06, 0x00, 0x02, 0x00, 0x01, 0x00, 
        0xfe, 0x00, 0x00, 0x00, 
        0x24, 
        0x01, 0x01, 0xff, 0x00, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 
        0x03, 0x04, 0xff, 0x00, 0x07, 0x02, 0x03, 0x02, 0x01, 0x00, 
        0x03, 0x04, 0xff, 0x00, 0x08, 0x02, 0x01, 0x02, 0x01, 0x00, 
        0x03, 0x04, 0xff, 0x00, 0x08, 0x02, 0x01, 0x02, 0x01, 0x02, 
        0x03, 0x04, 0xff, 0x00, 0x08, 0x02, 0x01, 0x02, 0x01, 0x04, 
        0x03, 0x19, 0xff, 0x00, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 
        0x03, 0x1c, 0xff, 0x00, 0x07, 0x02, 0x03, 0x02, 0x01, 0x00, 
        0x03, 0xda, 0xff, 0x00, 0x01, 0x00, 0x01, 0x02, 0x00, 0x00, 
        0x04, 0xc1, 0x00, 0x00, 0x02, 0x06, 0x01, 0x02, 0x01, 0x00, 
        0x04, 0xc1, 0x01, 0x00, 0x02, 0x06, 0x01, 0x02, 0x01, 0x00, 
        0x04, 0xc1, 0x02, 0x00, 0x02, 0x06, 0x01, 0x02, 0x01, 0x00, 
        0x04, 0xc1, 0x03, 0x00, 0x02, 0x06, 0x01, 0x02, 0x01, 0x00, 
        0x11, 0x0a, 0xff, 0x00, 
        0x11, 0xcc, 0xff, 0x00, 0x0e, 0x00, 0x01, 0x03, 0x00, 0x00, 
        0x11, 0xcc, 0xff, 0x00, 0x01, 0x00, 0x01, 0x02, 0x00, 0x01, 
        0x11, 0xcc, 0xff, 0x00, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 
        0x12, 0x01, 0xff, 0x00, 0x03, 0x03, 0x01, 0x04, 0x00, 0x00, 
        0x12, 0x02, 0xff, 0x00, 0x04, 0x03, 0x01, 0x04, 0x01, 0x00, 
        0x12, 0xc1, 0xff, 0x00, 0x03, 0x03, 0x01, 0x04, 0x00, 0x00, 
        0x13, 0x05, 0xff, 0x00, 0x05, 0x04, 0x03, 0x02, 0x01, 0x00, 
        0x13, 0x05, 0xff, 0x00, 0x06, 0x04, 0x01, 0x02, 0x01, 0x00, 
        0x13, 0x05, 0xff, 0x00, 0x06, 0x04, 0x01, 0x02, 0x01, 0x02, 
        0x13, 0x05, 0xff, 0x00, 0x06, 0x04, 0x01, 0x02, 0x01, 0x04, 
        0x13, 0x07, 0xff, 0x00, 0x05, 0x04, 0x03, 0x02, 0x01, 0x00, 
        0x14, 0x03, 0xff, 0x00, 0x01, 0x00, 0x01, 0x04, 0x00, 0x00, 
        0x15, 0x05, 0xff, 0x00, 0x0c, 0x08, 0x03, 0x02, 0x01, 0x00, 
        0x15, 0x05, 0xff, 0x00, 0x0d, 0x08, 0x01, 0x02, 0x01, 0x00, 
        0x15, 0x05, 0xff, 0x00, 0x0d, 0x08, 0x01, 0x02, 0x01, 0x02, 
        0x15, 0x05, 0xff, 0x00, 0x0d, 0x08, 0x01, 0x02, 0x01, 0x04, 
        0x19, 0x04, 0xff, 0x00, 0x1a, 0x00, 0x01, 0x0d, 0x01, 0x00, 
        0x19, 0x05, 0xff, 0x00, 0x18, 0x00, 0x01, 0x0d, 0x01, 0x00, 
        0x19, 0x06, 0xff, 0x00, 0x18, 0x00, 0x01, 0x0d, 0x01, 0x00, 
        0x19, 0x07, 0xff, 0x00, 0x16, 0x00, 0x04, 0x04, 0x01, 0x00, 
        0x19, 0x08, 0xff, 0x00, 0x17, 0x00, 0x04, 0x04, 0x01, 0x00, 
        0x19, 0x09, 0xff, 0x00, 0x19, 0x00, 0x03, 0x04, 0x01, 0x00, 
        0x19, 0x0a, 0xff, 0x00, 0x19, 0x00, 0x03, 0x04, 0x01, 0x00, 
        0x05, 
        0x03, 0x28, 0x03, 0x07, 0x30, 0x81, 0x0b, 0xc0, 0x00, 0x14, 0x14, 0x14, 0x40, 0x0a, 0x18, 0x48, 0x08, 0x11, 0x00, 0x00, 
        0x12, 0x2c, 0x00, 
        0x13, 0x28, 0x00, 
        0x14, 0x00, 0x03, 
        0x19, 0x00, 0x03, 0x00, 
        0x01, 
        0xb2, 0xb5, 0x3b, 0x95, 0x58, 0x01, 0x00, 0x00, 0x26, 0x92, 0x00, 0x00, 0x05, 
        0x00
    ]


    def setUp(self):
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

    def test_serialize_motion_r(self):
        self.firmware_revision= create_string_buffer(b'1.3.1', 5)
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        state_array_size= c_uint(0)
        state_ptr= cast(self.libmetawear.mbl_mw_metawearboard_serialize(self.board, byref(state_array_size)), POINTER(c_ubyte * state_array_size.value))

        self.python_array= []
        for i in range(0, state_array_size.value):
            self.python_array.append(state_ptr.contents[i])
        self.libmetawear.mbl_mw_memory_free(state_ptr)

        self.maxDiff= None
        self.assertEqual(self.python_array[0:518], TestMetaWearBoardSerialize.motion_r_state[0:518])

    def test_deserialize_motion_r(self):
        # just test that deserialization is successful
        state_buffer= self.to_string_buffer(TestMetaWearBoardSerialize.motion_r_state)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer.raw, len(state_buffer.raw))

        self.assertTrue(True)

    def test_deserialize_motion_r_new_fw(self):
        firmware_state = [0x03, 0x03, 0x01]

        self.firmware_revision= create_string_buffer(b'1.3.3', 5)
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD

        state_buffer= self.to_string_buffer(TestMetaWearBoardSerialize.motion_r_state)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer.raw, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        state_array_size= c_uint(0)
        state_ptr= cast(self.libmetawear.mbl_mw_metawearboard_serialize(self.board, byref(state_array_size)), POINTER(c_ubyte * state_array_size.value))

        python_array= []
        for i in range(0, state_array_size.value):
            python_array.append(state_ptr.contents[i])
        self.libmetawear.mbl_mw_memory_free(state_ptr)

        self.assertEqual(python_array[1:4], firmware_state)

    def test_serialize_readable_logger(self):
        logger_state= [
            0x1,
            0xff, 0x04, 0xc1, 0x00, 0x01, 0x00
        ]

        self.firmware_revision= create_string_buffer(b'1.2.5', 5)
        self.boardType = TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        readable_signal = self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, 0)
        self.libmetawear.mbl_mw_datasignal_log(readable_signal, self.logger_created);

        state_array_size= c_uint(0)
        state_ptr= cast(self.libmetawear.mbl_mw_metawearboard_serialize(self.board, byref(state_array_size)), POINTER(c_ubyte * state_array_size.value))

        python_array= []
        for i in range(0, state_array_size.value):
            python_array.append(state_ptr.contents[i])
        self.libmetawear.mbl_mw_memory_free(state_ptr)

        print(python_array)
        self.assertEqual(python_array[368:], logger_state)

    def test_deserialize_readable_logger(self):
        state = [
            1, 
            5, 2, 1, 
            1, 49, 
            25, 
            1, 0, 0, 0, 
            2, 0, 0, 0, 
            3, 1, 1, 0, 
            4, 1, 0, 4, 0, 3, 1, 2, 
            5, 0, 0, 0, 
            6, 0, 0, 0, 
            7, 0, 0, 0, 
            8, 0, 0, 0, 
            9, 0, 0, 1, 28, 
            10, 0, 0, 1, 28, 
            11, 0, 2, 5, 8, 128, 45, 0, 0, 
            12, 0, 0, 1, 8, 
            13, 0, 0, 0, 
            15, 0, 0, 0, 
            16, 255, 255, 
            17, 0, 0, 0, 
            18, 0, 0, 0, 
            19, 0, 1, 0, 
            20, 0, 0, 0, 
            21, 255, 255, 
            22, 255, 255, 
            23, 255, 255, 
            24, 255, 255, 
            25, 255, 255, 
            254, 0, 0, 0, 
            21, 
            1, 1, 255, 0, 1, 0, 1, 1, 0, 0, 
            3, 4, 255, 0, 7, 2, 3, 2, 1, 0, 
            3, 4, 255, 0, 8, 2, 1, 2, 1, 0, 
            3, 4, 255, 0, 8, 2, 1, 2, 1, 2, 
            3, 4, 255, 0, 8, 2, 1, 2, 1, 4, 
            3, 25, 255, 0, 1, 0, 1, 1, 0, 0, 
            3, 28, 255, 0, 7, 2, 3, 2, 1, 0, 
            3, 218, 255, 0, 1, 0, 1, 2, 0, 0, 
            4, 193, 0, 0, 2, 6, 1, 2, 1, 0, 
            4, 193, 1, 0, 2, 6, 1, 2, 1, 0, 
            4, 193, 2, 0, 2, 6, 1, 2, 1, 0, 
            4, 193, 3, 0, 2, 6, 1, 2, 1, 0, 
            18, 1, 255, 0, 3, 3, 1, 4, 0, 0, 
            18, 2, 255, 0, 4, 3, 1, 4, 1, 0, 
            18, 193, 255, 0, 3, 3, 1, 4, 0, 0, 
            19, 5, 255, 0, 5, 4, 3, 2, 1, 0, 
            19, 5, 255, 0, 6, 4, 1, 2, 1, 0, 
            19, 5, 255, 0, 6, 4, 1, 2, 1, 2, 
            19, 5, 255, 0, 6, 4, 1, 2, 1, 4, 
            19, 7, 255, 0, 5, 4, 3, 2, 1, 0, 
            20, 3, 255, 0, 1, 0, 1, 4, 0, 0, 
            4, 
            3, 40, 3, 7, 48, 129, 11, 192, 0, 20, 20, 20, 64, 10, 24, 72, 8, 17, 0, 0, 
            18, 44, 0, 
            19, 40, 0, 
            20, 0, 3, 
            1, 
            226, 118, 142, 47, 90, 1, 0, 0, 21, 4, 0, 0, 5, 
            1, 
            255, 4, 193, 0, 1, 0
        ]

        self.firmware_revision= create_string_buffer(b'1.2.5', 5)
        self.boardType = TestMetaWearBase.METAWEAR_RPRO_BOARD

        state_buffer= self.to_string_buffer(state)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer.raw, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        logger = self.libmetawear.mbl_mw_logger_lookup_id(self.board, 0);
        self.libmetawear.mbl_mw_logger_subscribe(logger, self.sensor_data_handler);

        buffer = create_string_buffer(b'\x0b\x07\xa0\x37\x43\x00\x00\xc8\x00\x00\x00', 11)
        self.libmetawear.mbl_mw_metawearboard_notify_char_changed(self.board, buffer.raw, len(buffer))

        self.assertAlmostEqual(self.data_float.value, 25.0)

class TestMetaWearBoardTearDownSerialize(TestMetaWearBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.expected_state= [
            0x01, 
            0x03, 0x01, 0x01, 
            0x01, 0x31, 
            0x18, 
            0x01, 0xff, 0xff,
            0x02, 0xff, 0xff,
            0x03, 0x01, 0x00, 0x00, 
            0x04, 0xff, 0xff,
            0x05, 0xff, 0xff,
            0x06, 0xff, 0xff,
            0x07, 0xff, 0xff,
            0x08, 0xff, 0xff,
            0x09, 0xff, 0xff,
            0x0a, 0xff, 0xff,
            0x0b, 0xff, 0xff,
            0x0c, 0xff, 0xff,
            0x0d, 0xff, 0xff,
            0x0f, 0xff, 0xff,
            0x10, 0xff, 0xff,
            0x11, 0x00, 0x03, 0x00,
            0x12, 0xff, 0xff,
            0x13, 0xff, 0xff,
            0x14, 0xff, 0xff,
            0x15, 0xff, 0xff,
            0x16, 0xff, 0xff,
            0x17, 0xff, 0xff,
            0x18, 0xff, 0xff,
            0xfe, 0xff, 0xff,
            0x0a, 
            0x03, 0x04, 0xff, 0x00, 0x07, 0x02, 0x03, 0x02, 0x01, 0x00,
            0x03, 0x04, 0xff, 0x00, 0x08, 0x02, 0x01, 0x02, 0x01, 0x00, 
            0x03, 0x04, 0xff, 0x00, 0x08, 0x02, 0x01, 0x02, 0x01, 0x02, 
            0x03, 0x04, 0xff, 0x00, 0x08, 0x02, 0x01, 0x02, 0x01, 0x04,  
            0x03, 0x19, 0xff, 0x00, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00,
            0x03, 0x9a, 0xff, 0x00, 0x01, 0x00, 0x01, 0x02, 0x00, 0x00,
            0x11, 0x0a, 0xff, 0x00, 
            0x11, 0x8c, 0xff, 0x00, 0x0e, 0x00, 0x01, 0x03, 0x00, 0x00,
            0x11, 0x8c, 0xff, 0x00, 0x0e, 0x00, 0x01, 0x02, 0x00, 0x01,
            0x11, 0x8c, 0xff, 0x00, 0x0e, 0x00, 0x01, 0x01, 0x00, 0x00,
            0x01,
            0x03, 0x28, 0x03, 0x07, 0x30, 0x81, 0x0b, 0xc0, 0x00, 0x14, 0x14, 0x14, 0x40, 0x0a, 0x18, 0x48, 0x08, 0x11, 0x00, 0x00, 
            0x01, 
            0xf6, 0x0b, 0x99, 0x36, 0x55, 0x01, 0x00, 0x00, 0x15, 0x04, 0x00, 0x00, 0x05, 
            0x00
        ]

    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))
        self.logger_ready= Fn_VoidPtr(self.logger_ready_handler)

    def logger_ready_handler(self, logger):
        self.libmetawear.mbl_mw_metawearboard_tear_down(self.board)

        state_array_size= c_uint(0)
        state_ptr= cast(self.libmetawear.mbl_mw_metawearboard_serialize(self.board, byref(state_array_size)), POINTER(c_ubyte * state_array_size.value))

        self.python_array= []
        for i in range(0, state_array_size.value):
            self.python_array.append(state_ptr.contents[i])
        self.libmetawear.mbl_mw_memory_free(state_ptr)

    def test_teardown_serialize(self):
        state_buffer= self.to_string_buffer(self.expected_state)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer.raw, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        acc_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_log(acc_signal, self.logger_ready)

        self.maxDiff= None
        self.assertEqual(self.python_array[0:198], self.expected_state[0:198])

class TestMetaWearBoardDeserialize(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.state= self.to_string_buffer(serializedstate.activity_with_buffer)
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

    def test_deserialize_same_firmware(self):
        # only thing to do is re-read the logging tick
        expected= [0xb, 0x84]

        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, self.state.raw, len(self.state.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        self.assertEqual(self.command, expected)

    def test_deserialize_diff_firmware(self):
        expected_cmds= [
            [0x01, 0x80], [0x02, 0x80], [0x03, 0x80], [0x04, 0x80],
            [0x05, 0x80], [0x06, 0x80], [0x07, 0x80], [0x08, 0x80],
            [0x09, 0x80], [0x0a, 0x80], [0x0b, 0x80], [0x0c, 0x80],
            [0x0d, 0x80], [0x0f, 0x80], [0x10, 0x80], [0x11, 0x80],
            [0x12, 0x80], [0x13, 0x80], [0x14, 0x80], [0x15, 0x80],
            [0x16, 0x80], [0x17, 0x80], [0x18, 0x80], [0x19, 0x80],
            [0xfe, 0x80], [0x0b, 0x84]
        ]

        self.firmware_revision= create_string_buffer(b'1.0.4', 5)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, self.state.raw, len(self.state.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        self.assertEqual(self.full_history, expected_cmds)

class TestDeserializeTimer(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

        state_buffer= self.to_string_buffer(serializedstate.timer_two_events)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

        self.serialized_timer= self.libmetawear.mbl_mw_timer_lookup_id(self.board, 7);

    def test_start(self):
        expected= [0x0c, 0x03, 0x7]

        self.libmetawear.mbl_mw_timer_start(self.serialized_timer)
        self.assertEqual(self.command, expected)

    def test_stop(self):
        expected= [0x0c, 0x04, 0x7]

        self.libmetawear.mbl_mw_timer_stop(self.serialized_timer)
        self.assertEqual(self.command, expected)

    def test_remove(self):
        expected_cmds= [
            [0x0c, 0x05, 0x7],
            [0x0a, 0x04, 0x0],
            [0x0a, 0x04, 0x1]
        ]

        self.libmetawear.mbl_mw_timer_remove(self.serialized_timer)
        self.assertEqual(self.command_history, expected_cmds)

    def test_null_timer(self):
        invalid_ids= [0, 1, 2, 3, 4, 5, 6]

        for id in invalid_ids:
            with self.subTest(timer_id=id):
                self.assertIsNone(self.libmetawear.mbl_mw_timer_lookup_id(self.board, id))

class TestDeserializeAccelerometerLog(TestAccelerometerLoggingBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

        state_buffer= self.to_string_buffer(serializedstate.accelerometer_log)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)
        self.acc_logger= self.libmetawear.mbl_mw_logger_lookup_id(self.board, 0)

    def test_acc_data(self):
        self.logger_ready_handler(self.acc_logger)
        self.assertEqual(self.logged_data, Bmi160Accelerometer.expected_values)

    def test_epoch_calc(self):
        self.logger_ready_handler(self.acc_logger)
        self.assertEqual(self.data_time_offsets, Bmi160Accelerometer.expected_offsets)

    def test_remove_logger(self):
        expected_cmds= [
            [0x0b, 0x03, 0x00],
            [0x0b, 0x03, 0x01]
        ]

        self.libmetawear.mbl_mw_logger_remove(self.acc_logger)
        self.assertEqual(self.command_history, expected_cmds)

class TestDeserializeGyroYAxisLog(TestGyroYAxisLoggingBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

        state_buffer= self.to_string_buffer(serializedstate.gyro_y_axis_logging_state)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)
        self.gyro_y_logger= self.libmetawear.mbl_mw_logger_lookup_id(self.board, 0)

    def test_gyro_data(self):
        self.logger_ready_handler(self.gyro_y_logger)

        # why doesn't unittest come with an assertAlmostEqual for list of floats?
        self.assertEqual(len(self.logged_data), len(Bmi160GyroYAxis.expected_values))
        for a, b in zip(self.logged_data, Bmi160GyroYAxis.expected_values):
            self.assertAlmostEqual(a, b, delta = 0.001)

    def test_remove_logger(self):
        expected_cmds= [
            [0x0b, 0x03, 0x00]
        ]

        self.libmetawear.mbl_mw_logger_remove(self.gyro_y_logger)
        self.assertEqual(self.command_history, expected_cmds)

class TestDeserializeActivityHandler(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

        state_buffer= self.to_string_buffer(serializedstate.activity_with_buffer)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

    def test_read_state(self):
        ids= [1, 2]

        for id in ids:
            with self.subTest(processor_id=id):
                processor= self.libmetawear.mbl_mw_dataprocessor_lookup_id(self.board, id)
                state_signal= self.libmetawear.mbl_mw_dataprocessor_get_state_data_signal(processor)
                self.libmetawear.mbl_mw_datasignal_read(state_signal)

                expected= [0x9, 0xc4, id]
                self.assertEqual(self.command, expected)

    def test_remove_processor_chain(self):
        expected_cmds= [
            [0x9, 0x6, 0x2],
            [0x9, 0x6, 0x4],
            [0x9, 0x6, 0x3],
            [0x9, 0x6, 0x1]
        ]

        processor= self.libmetawear.mbl_mw_dataprocessor_lookup_id(self.board, 1)
        self.libmetawear.mbl_mw_dataprocessor_remove(processor)
        self.assertEqual(self.command_history, expected_cmds)

    def test_buffer_read(self):
        response= create_string_buffer(b'\x09\x84\x02\xcd\x20\x41\x00', 7)
        expected= 260.5125

        processor= self.libmetawear.mbl_mw_dataprocessor_lookup_id(self.board, 2)
        processor_state= self.libmetawear.mbl_mw_dataprocessor_get_state_data_signal(processor)
        self.libmetawear.mbl_mw_datasignal_subscribe(processor_state, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))

        self.assertAlmostEqual(self.data_float.value, expected, delta= 0.0001)

    def test_timer_data(self):
        response= create_string_buffer(b'\x09\x03\x03\xfe\x3f\xb7\x01', 7)
        expected= 1756.9999

        processor= self.libmetawear.mbl_mw_dataprocessor_lookup_id(self.board, 3)
        self.libmetawear.mbl_mw_datasignal_subscribe(processor, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))

        self.assertAlmostEqual(self.data_float.value, expected, delta= 0.0001)

class TestSerializeMultiComparator(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.firmware_revision= create_string_buffer(b'1.2.3', 5)

        super().setUp()

    def test_initial_state(self):
        references= (c_float * 4)(1024, 512, 256, 128)
        adc_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0x15, Gpio.ANALOG_READ_MODE_ADC)

        comp_created_ptr= Fn_VoidPtr(self.comp_created_cmd_check)
        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_create(adc_signal, Comparator.OPERATION_GTE, Comparator.MODE_REFERENCE, references, 
                len(references), comp_created_ptr)

        self.assertEqual(self.python_array[0:388], serializedstate.multi_comparator_state[0:388])

    def comp_created_cmd_check(self, comparator):
        state_array_size= c_uint(0)
        state_ptr= cast(self.libmetawear.mbl_mw_metawearboard_serialize(self.board, byref(state_array_size)), POINTER(c_ubyte * state_array_size.value))

        self.python_array= []
        for i in range(0, state_array_size.value):
            self.python_array.append(state_ptr.contents[i])
        self.libmetawear.mbl_mw_memory_free(state_ptr)

class TestDeserializeMultiComparator(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        self.firmware_revision= create_string_buffer(b'1.2.3', 5)
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))

        state_buffer= self.to_string_buffer(serializedstate.multi_comparator_state)
        self.libmetawear.mbl_mw_metawearboard_deserialize(self.board, state_buffer, len(state_buffer.raw))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

    def test_modify_config_state(self):
        references= (c_float * 2)(128, 256)

        comparator= self.libmetawear.mbl_mw_dataprocessor_lookup_id(self.board, 0)
        self.libmetawear.mbl_mw_dataprocessor_multi_comparator_modify(comparator, Comparator.OPERATION_LT, references, len(references))

        state_array_size= c_uint(0)
        state_ptr= cast(self.libmetawear.mbl_mw_metawearboard_serialize(self.board, byref(state_array_size)), POINTER(c_ubyte * state_array_size.value))

        python_array= []
        for i in range(0, state_array_size.value):
            python_array.append(state_ptr.contents[i])
        self.libmetawear.mbl_mw_memory_free(state_ptr)

        self.assertEqual(python_array[0:378], serializedstate.multi_comparator_modified_state[0:378])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging
from functools import wraps
from ctypes import c_float, cast, POINTER

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear import sensor
from pymetawear.mbientlab.metawear.core import DataTypeId, CartesianFloat
from pymetawear.modules.base import PyMetaWearModule

log = logging.getLogger(__name__)


class AccelerometerModule(PyMetaWearModule):
    """MetaWear accelerometer module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this accelerometer
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id, debug=False):
        super(AccelerometerModule, self).__init__(board, debug)
        self.module_id = module_id

        self.high_frequency_stream = False

        acc_sensors = [
            sensor.AccelerometerBmi160,
            sensor.AccelerometerBma255,
            sensor.AccelerometerMma8452q
        ]
        for a in acc_sensors:
            if getattr(a, 'MODULE_TYPE', -1) == module_id:
                self.acc_class = a

        if self.acc_class is not None:
            # Parse possible output data rates for this accelerometer.
            self.odr = {".".join(re.search(
                '^ODR_([0-9]+)\_*([0-9]*)HZ', k).groups()):
                getattr(self.acc_class, k, None) for k in filter(
                lambda x: x.startswith('ODR'), vars(self.acc_class))}

            # Parse possible output data ranges for this accelerometer.
            if len(list(filter(lambda x: x.startswith('FSR'),
                               vars(self.acc_class)))) == 0:
                acc_class = sensor.AccelerometerBosch
            else:
                acc_class = self.acc_class
            self.fsr = {float(re.search('^FSR_([0-9]+)G', k).groups()[0]):
                        getattr(acc_class, k, None) for k in
                        filter(lambda x: x.startswith('FSR'), vars(acc_class))}

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0} {1}: Data rates (Hz): {2}, Data ranges (g): {3}".format(
            self.module_name, self.sensor_name,
            [float(k) for k in sorted(self.odr.keys(),
                                      key=lambda x:(float(x)))],
            [k for k in sorted(self.fsr.keys())])

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Accelerometer"

    @property
    def sensor_name(self):
        return self.acc_class.__name__.replace('Accelerometer', '')

    @property
    def data_signal(self):
        if self.high_frequency_stream:
            return libmetawear.mbl_mw_acc_get_high_freq_acceleration_data_signal(self.board)
        else:
            return libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)

    def _get_odr(self, value):
        sorted_ord_keys = sorted(self.odr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested ODR ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        return float(value)

    def _get_fsr(self, value):
        sorted_ord_keys = sorted(self.fsr.keys(), key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.1:
            raise ValueError(
                "Requested FSR ({0}) was not part of possible values: {1}".format(
                    value, [x for x in sorted(self.fsr.keys())]))
        return float(value)

    def get_current_settings(self):
        raise NotImplementedError()

    def get_possible_settings(self):
        return {
            'data_rate': [float(x) for x in sorted(
                self.odr.keys(), key=lambda x:(float(x)))],
            'data_range': [x for x in sorted(self.fsr.keys())]
        }

    def set_settings(self, data_rate=None, data_range=None):
        """Set accelerometer settings.

         Can be called with two or only one setting:

         .. code-block:: python

            mwclient.set_accelerometer_settings(data_rate=200.0, data_range=8.0)

        will give the same result as

        .. code-block:: python

            mwclient.set_accelerometer_settings(data_rate=200.0)
            mwclient.set_accelerometer_settings(data_range=8.0)

        albeit that the latter example makes two writes to the board.

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param float data_rate: The frequency of accelerometer updates in Hz.
        :param float data_range: The measurement range in the unit ``g``.

        """

        if data_rate is not None:
            odr = self._get_odr(data_rate)
            if self._debug:
                log.debug("Setting Accelerometer ODR to {0}".format(odr))
            libmetawear.mbl_mw_acc_set_odr(self.board, c_float(odr))
        if data_range is not None:
            fsr = self._get_fsr(data_range)
            if self._debug:
                log.debug("Setting Accelerometer FSR to {0}".format(fsr))
            libmetawear.mbl_mw_acc_set_range(self.board, c_float(fsr))

        if (data_rate is not None) or (data_range is not None):
            libmetawear.mbl_mw_acc_write_acceleration_config(self.board)

    def notifications(self, callback=None):
        """Subscribe or unsubscribe to accelerometer notifications.

        Convenience method for handling accelerometer usage.

        Example:

        .. code-block:: python

            def handle_acc_notification(data)
                # Handle a (epoch_time, (x,y,z)) accelerometer tuple.
                epoch = data[0]
                xyz = data[1]
                print("[{0}] X: {1}, Y: {2}, Z: {3}".format(epoch, *xyz))

            mwclient.accelerometer.notifications(handle_acc_notification)

        :param callable callback: Accelerometer notification callback function.
            If `None`, unsubscription to accelerometer notifications is registered.

        """

        if callback is None:
            self.stop()
            self.toggle_sampling(False)
            super(AccelerometerModule, self).notifications(None)
        else:
            super(AccelerometerModule, self).notifications(
                sensor_data(callback))
            self.toggle_sampling(True)
            self.start()

    def start(self):
        """Switches the accelerometer to active mode."""
        libmetawear.mbl_mw_acc_start(self.board)

    def stop(self):
        """Switches the accelerometer to standby mode."""
        libmetawear.mbl_mw_acc_stop(self.board)

    def toggle_sampling(self, enabled=True):
        """Enables or disables accelerometer sampling.

        :param bool enabled: Desired state of the accelerometer.

        """
        if enabled:
            libmetawear.mbl_mw_acc_enable_acceleration_sampling(self.board)
        else:
            libmetawear.mbl_mw_acc_disable_acceleration_sampling(self.board)


def sensor_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.CARTESIAN_FLOAT:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(CartesianFloat))
            func((epoch, (data_ptr.contents.x,
                          data_ptr.contents.y,
                          data_ptr.contents.z)))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))
    return wrapper



from common import TestMetaWearBase
from mbientlab.metawear.sensor import SensorFusion

class TestSensorFusionConfig(TestMetaWearBase):
    acc_ranges = [SensorFusion.ACC_RANGE_2G, SensorFusion.ACC_RANGE_4G, SensorFusion.ACC_RANGE_8G, SensorFusion.ACC_RANGE_16G]
    gyro_ranges = [SensorFusion.GYRO_RANGE_2000DPS, SensorFusion.GYRO_RANGE_1000DPS, SensorFusion.GYRO_RANGE_500DPS, SensorFusion.GYRO_RANGE_250DPS]
    config_masks = [
        [0x10, 0x11, 0x12, 0x13],
        [0x20, 0x21, 0x22, 0x23],
        [0x30, 0x31, 0x32, 0x33],
        [0x40, 0x41, 0x42, 0x43]
    ]
    bmi160_acc_range_bitmask= [ 0b0011, 0b0101, 0b1000, 0b1100 ]
    bmi160_rot_range_bitmask= [ 0b000, 0b001, 0b010, 0b011, 0b100 ];


    def setUp(self):
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD

        super().setUp()

    def queue_tests():
        tests= []

        for ar in TestSensorFusionConfig.acc_ranges:
            for gr in TestSensorFusionConfig.gyro_ranges:
                tests.append({
                    'acc_range': ar,
                    'gyro_range': gr
                })

        return tests

    def configure_algorithm(self, mode, acc, gyro):
        self.libmetawear.mbl_mw_sensor_fusion_set_mode(self.board, mode)
        self.libmetawear.mbl_mw_sensor_fusion_set_acc_range(self.board, acc)
        self.libmetawear.mbl_mw_sensor_fusion_set_gyro_range(self.board, gyro)
        self.libmetawear.mbl_mw_sensor_fusion_write_config(self.board)

    def test_configure_ndof(self):
        for test in TestSensorFusionConfig.queue_tests():
            with self.subTest():
                self.command_history = []
                self.configure_algorithm(SensorFusion.MODE_NDOF, test['acc_range'], test['gyro_range'])
                expected = [
                    [0x19, 0x02, SensorFusion.MODE_NDOF, TestSensorFusionConfig.config_masks[test['gyro_range']][test['acc_range']]],
                    [0x03, 0x03, 0x28, TestSensorFusionConfig.bmi160_acc_range_bitmask[test['acc_range']]],
                    [0x13, 0x03, 0x28, TestSensorFusionConfig.bmi160_rot_range_bitmask[test['gyro_range']]],
                    [0x15, 0x04, 0x04, 0x0e],
                    [0x15, 0x03, 0x6]
                ]
                self.assertEqual(self.command_history, expected)

    def test_configure_imu_plus(self):
        for test in TestSensorFusionConfig.queue_tests():
            with self.subTest():
                self.command_history = []
                self.configure_algorithm(SensorFusion.MODE_IMU_PLUS, test['acc_range'], test['gyro_range'])
                expected = [
                    [0x19, 0x02, SensorFusion.MODE_IMU_PLUS, TestSensorFusionConfig.config_masks[test['gyro_range']][test['acc_range']]],
                    [0x03, 0x03, 0x28, TestSensorFusionConfig.bmi160_acc_range_bitmask[test['acc_range']]],
                    [0x13, 0x03, 0x28, TestSensorFusionConfig.bmi160_rot_range_bitmask[test['gyro_range']]],
                ]
                self.assertEqual(self.command_history, expected)

    def test_configure_compass(self):
        for test in TestSensorFusionConfig.queue_tests():
            with self.subTest():
                self.command_history = []
                self.configure_algorithm(SensorFusion.MODE_COMPASS, test['acc_range'], test['gyro_range'])
                expected = [
                    [0x19, 0x02, SensorFusion.MODE_COMPASS, TestSensorFusionConfig.config_masks[test['gyro_range']][test['acc_range']]],
                    [0x03, 0x03, 0x26, TestSensorFusionConfig.bmi160_acc_range_bitmask[test['acc_range']]],
                    [0x15, 0x04, 0x04, 0x0e],
                    [0x15, 0x03, 0x6]
                ]
                self.assertEqual(self.command_history, expected)

    def test_configure_compass(self):
        for test in TestSensorFusionConfig.queue_tests():
            with self.subTest():
                self.command_history = []
                self.configure_algorithm(SensorFusion.MODE_M4G, test['acc_range'], test['gyro_range'])
                expected = [
                    [0x19, 0x02, SensorFusion.MODE_M4G, TestSensorFusionConfig.config_masks[test['gyro_range']][test['acc_range']]],
                    [0x03, 0x03, 0x27, TestSensorFusionConfig.bmi160_acc_range_bitmask[test['acc_range']]],
                    [0x15, 0x04, 0x04, 0x0e],
                    [0x15, 0x03, 0x6]
                ]
                self.assertEqual(self.command_history, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created on 2016-03-30

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import logging
import os

from pymetawear import libmetawear, specs, add_stream_logger
from pymetawear import modules
from pymetawear.backends.pybluez import PyBluezBackend
from pymetawear.backends.pygatt import PyGattBackend
from pymetawear.exceptions import PyMetaWearException, PyMetaWearConnectionTimeout
from pymetawear.mbientlab.metawear.core import Status

log = logging.getLogger(__name__)


class MetaWearClient(object):
    """A MetaWear communication client.

    This client bridges the gap between the
    `MetaWear C++ API <https://github.com/mbientlab/Metawear-CppAPI>`_
    and a GATT communication package in Python. It provides Pythonic
    interface to using the MetaWear boards, allowing for rapid
    development and testing.

    :param str address: A Bluetooth MAC address to a MetaWear board.
    :param str backend: Either ``pygatt`` or ``pybluez``, designating which
        BLE communication backend that should be used.
    :param float timeout: Timeout for connecting to the MetaWear board. If
        ``None`` timeout defaults to the backend default.
    :param bool connect: If client should connect automatically, or wait for
        explicit :py:meth:`~MetaWearClient.connect` call. Default is ``True``.
    :param bool debug: If printout of all sent and received
        data should be done.

    """

    def __init__(self, address, backend='pygatt',
                 interface='hci0', timeout=None, connect=True, debug=False):
        """Constructor."""
        self._address = address
        self._debug = debug
        self._initialized = False

        if self._debug:
            add_stream_logger()
            log.info("Creating MetaWearClient for {0}...".format(address))

        # Handling of timeout.
        if timeout is None:
            timeout = os.environ.get('PYMETAWEAR_TIMEOUT', None)
            if timeout is not None:
                try:
                    timeout = float(timeout)
                except:
                    timeout = None

        if backend == 'pygatt':
            self._backend = PyGattBackend(
                self._address, interface=interface,
                timeout=timeout, debug=debug)
        elif backend == 'pybluez':
            self._backend = PyBluezBackend(
                self._address, interface=interface,
                timeout=timeout, debug=debug)
        else:
            raise PyMetaWearException("Unknown backend: {0}".format(backend))

        self.firmware_version = None
        self.model_version = None
        self.accelerometer = None
        self.gyroscope = None
        self.magnetometer = None
        self.barometer = None
        self.ambient_light = None
        self.switch = None
        self.battery = None
        self.temperature = None
        self.haptic = None
        self.led = None
        self.sensorfusion = None

        if connect:
            self.connect()

    def __str__(self):
        return "MetaWearClient, {0}, Model: {1}, Firmware: {2}".format(
            self.backend, self.model_version,
            ".".join([str(i) for i in self.firmware_version]) if self.firmware_version else None)

    def __repr__(self):
        return "<MetaWearClient, {0}>".format(self._address)

    @property
    def backend(self):
        """The backend object for this client.

        :return: The connected BLE backend.
        :rtype: :class:`pymetawear.backend.BLECommunicationBackend`

        """
        return self._backend

    @property
    def board(self):
        return self.backend.board

    def connect(self, clean_connect=False):
        """Connect this client to the MetaWear device.

        :param bool clean_connect: If old backend components should be replaced.
            Default is ``False``.

        """
        if self.backend.is_connected:
            return
        self.backend.connect(clean_connect=clean_connect)

        if self._debug:
            log.debug("Waiting for MetaWear board to be fully initialized...")

        while not self.backend.initialized:
            self.backend.sleep(0.1)

        # Check if initialization has been completed successfully.
        if self.backend.initialization_status != Status.OK:
            if self.backend.initialization_status == Status.ERROR_TIMEOUT:
                raise PyMetaWearConnectionTimeout("libmetawear initialization status 16: Timeout")
            else:
                raise PyMetaWearException("libmetawear initialization status {0}".format(
                    self.backend.initialization_status))

        # Read out firmware and model version.
        self.firmware_version = tuple(
            [int(x) for x in self.backend.read_gatt_char_by_uuid(
                specs.DEV_INFO_FIRMWARE_CHAR[1]).decode().split('.')])
        self.model_version = int(self.backend.read_gatt_char_by_uuid(
            specs.DEV_INFO_MODEL_CHAR[1]).decode())

        self._initialize_modules()

    def disconnect(self):
        """Disconnects this client from the MetaWear device."""
        libmetawear.mbl_mw_metawearboard_tear_down(self.board)
        libmetawear.mbl_mw_metawearboard_free(self.board)
        self.backend.disconnect()

    def get_handle(self, uuid, notify_handle=False):
        """Get handle for a characteristic UUID.

        :param uuid.UUID uuid: The UUID to get handle of.
        :param bool notify_handle:
        :return: Integer handle corresponding to the input characteristic UUID.
        :rtype: int

        """
        return self.backend.get_handle(uuid, notify_handle=notify_handle)

    def _set_logging_state(self, enabled=False):
        if enabled:
            libmetawear.mbl_mw_logging_start(self.board)
        else:
            libmetawear.mbl_mw_logging_stop(self.board)

    def _download_log(self, n_notifies):
        libmetawear.mbl_mw_logging_download(self.board, n_notifies)

    def soft_reset(self):
        """Issues a soft reset to the board."""
        libmetawear.mbl_mw_debug_reset(self.board)

    def _initialize_modules(self):
        self.accelerometer = modules.AccelerometerModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_ACCELEROMETER),
            debug=self._debug)
        self.gyroscope = modules.GyroscopeModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_GYRO),
            debug=self._debug)
        self.magnetometer = modules.MagnetometerModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_MAGNETOMETER),
            debug=self._debug)
        self.barometer = modules.BarometerModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_BAROMETER),
            debug=self._debug)
        self.ambient_light = modules.AmbientLightModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_AMBIENT_LIGHT),
            debug=self._debug)
        self.switch = modules.SwitchModule(self.board, debug=self._debug)
        self.battery = modules.BatteryModule(self.board, debug=self._debug)
        self.temperature = modules.TemperatureModule(
            self.board, debug=self._debug)
        self.haptic = modules.HapticModule(self.board, debug=self._debug)
        self.led = modules.LEDModule(self.board, debug=self._debug)
        self.sensorfusion = modules.SensorFusionModule(
            self.board,
            libmetawear.mbl_mw_metawearboard_lookup_module(
                self.board, modules.Modules.MBL_MW_MODULE_SENSOR_FUSION),
            debug=self._debug)
